//****************************************************************************
// MMCTL.H
//
// Definitions and data types for multimedia PM controls
//
// Change History:
//
//****************************************************************************
#ifdef  INCL_MMCTL
#define INCL_VPC
#define INCL_AAC
#define INCL_VAC
#endif

// Common Messages
#define MCM_MESSAGE                    WM_USER + 500 // Change this later when stabilized

#define MCM_ENABLEMCI                  MCM_MESSAGE + 1
#define MCM_MCINOTIFY                  MCM_MESSAGE + 2
#define MCM_QUERYDEVICEALIAS           MCM_MESSAGE + 3
#define MCM_SETDEVICEALIAS             MCM_MESSAGE + 4
#define MCM_SETDLGTEMPLATE             MCM_MESSAGE + 5
#define MCM_UPDATE                     MCM_MESSAGE + 6

// Reserve 100 messages for each control
#define AAC_MESSAGE                    MCM_MESSAGE + 100
#define VAC_MESSAGE                    MCM_MESSAGE + 200
#define VPC_MESSAGE                    MCM_MESSAGE + 300

// Common Styles
//     Reserved by PM                  0xrrrr----
//     Reserved for specific controls  0x------rr
#define MCS_BORDER                     0x00000800
#define MCS_COMPLETE                   0x00004000
#define MCS_MINIMAL                    0x00008000
#define MCS_NOADJUSTPOS                0x00001000
#define MCS_VERTICAL                   0x00002000

// WM_CONTROL notification codes
#define MCN_ERROR                      0xFFFF

/*********************************************************************/
/* Videodisc player control definitions                              */
/*********************************************************************/
#ifdef INCL_VPC
#ifndef _VPC_   // Avoid multiple include
#define _VPC_
/*********************************************************************/
/* Videodisc player control Window Class                             */
/*********************************************************************/
BOOL EXPENTRY VpcRegisterClass( HAB hAB);

#define WC_PLAYERCTL "VIDEODISC PLAYER CONTROL"

/*********************************************************************/
/* Control Style bits                                                */
/*********************************************************************/
#define VPS_NOANIMATE       0x00000001

/*********************************************************************/
/* Control Id's - also used for WM_CONTROL notification code         */
/*********************************************************************/
#define ID_EJECT            100
#define ID_PAUSE            101
#define ID_PLAYR            102
#define ID_PLAY             103
#define ID_POSITION         104
#define ID_RECORD           105
#define ID_REPEAT           106
#define ID_REWIND           107
#define ID_SCANR            108
#define ID_SCAN             109
#define ID_STEPR            110
#define ID_STEP             111
#define ID_STOP             112
#define ID_POSTITLE         122

/*********************************************************************/
/* Control Message ID'S                                              */
/*********************************************************************/
#define VPM_SETPLAYRANGE     VPC_MESSAGE + 1
#define VPM_QUERYPLAYRANGE   VPC_MESSAGE + 2
#endif
#endif

/*********************************************************************/
/* Audio Attributes Control definitions                              */
/*********************************************************************/
#ifdef INCL_AAC
#ifndef _AAC_   // Avoid multiple include
#define _AAC_
/*********************************************************************/
/* Audio Control Window Class                                        */
/*********************************************************************/
BOOL EXPENTRY AACRegisterClass( HAB hAB);

#define WC_AUDIOCTL     "AUDIO CONTROL"

/*********************************************************************/
/* Control Id's - also used for WM_CONTROL notification code         */
/*********************************************************************/
#define AACREF                  3000
#define ID_MUTE                 (AACREF +  1)
#define ID_VOLUME               (AACREF +  2)
#define ID_BALANCE              (AACREF +  3)
#define ID_TREBLE               (AACREF +  4)
#define ID_BASS                 (AACREF +  5)

#endif // #ifndef
#endif // #ifdef

/*********************************************************************/
/* Video Attributes Control definitions                              */
/*********************************************************************/
#ifdef INCL_VAC
#ifndef _VAC_   // Avoid multiple include
#define _VAC_
/*********************************************************************/
/* Video Control Window Class                                        */
/*********************************************************************/
BOOL EXPENTRY VACRegisterClass( HAB hAB);

#define WC_VIDEOCTL     "VIDEO CONTROL"

#define VACREF                  4000

/*********************************************************************/
/* Control Id's - also used for WM_CONTROL notification code         */
/*********************************************************************/
#define ID_FREEZE               (VACREF +   1)
#define ID_BRIGHTNESS           (VACREF +   2)
#define ID_CONTRAST             (VACREF +   3)
#define ID_HUE                  (VACREF +   4)
#define ID_SATURATION           (VACREF +   5)
#define ID_SHARPNESS            (VACREF +   6)

#endif
#endif
