/*************************************************************************
*
*  File Name   : Devcaps.c
*
*  Description : This PM sample application demonstates the
*                the MCI String Command and Command Message
*                Interface. The application queries the system
*                for the installed MCI devices. All of the
*                found MCI devices can be queried for their
*                device capabilities ( MCI_GETDEVCAPS ).
*
*                The application creates a notebook which
*                presents the installed device names on the
*                Major tabs and the device capability information
*                on the notebook page for that device.
*
*                The MCI Command String Interface is used to query
*                all installed devices for the required device
*                capabilities. In addition, the Command Message
*                Interface is used to query the device specific 
*                capabilities for Overlay and Videodisc player
*                devices.
*
*
*
*  Concepts    : PM Notebook creation
*                Use of MMPM/2 Secondary Windows for creating sizable dialog
*                Query system for MCI device information
*                MCI Command String Interface
*                MCI Command Message Interface
*                Get device capabilities from installed devices
*
*  Copyright (C) 1992 IBM Corporation
*
*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
*      sample code created by IBM Corporation. This sample code is not
*      part of any standard or IBM product and is provided to you solely
*      for  the purpose of assisting you in the development of your
*      applications.  The code is provided "AS IS", without
*      warranty of any kind.  IBM shall not be liable for any damages
*      arising out of your use of the sample code, even if they have been
*      advised of the possibility of such damages.                                                    *
*
************************************************************************/

#define INCL_WIN
#define INCL_WINSYS                     /* Selectively include PM parts */
#define INCL_WINFRAMEMGR
#define INCL_WINMENUS
#define INCL_WINDIALOGS
#define INCL_WINSTDDLGS
#define INCL_GPI
#define INCL_SW
#define INCL_SECONDARYWINDOW

#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <os2me.h>
#include "devcaps.h"


#define LENGTH_STRING  33               /* Length of string             */
#define RETURN_ERROR    1               /* return value for exit        */
#define BEEP_WARN_FREQ      60          /* frequency of warning beep    */
#define BEEP_WARN_DUR      100          /* duration of warning beep     */

/************************************************/
/********** Function Prototype Section **********/
/************************************************/
int main( VOID );
MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY DevDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY Dev1InfoPageDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
MRESULT EXPENTRY DevHelpInfoDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 );
USHORT  SetupMyMainWindow ();
VOID DisplayMessage(PCH str);
BOOL SetupMyDevInfoPages( HWND hwnd );
BOOL GetDevSysInfo();
VOID ProcessDevCaps( HWND hwnd );
VOID VdiscSpecDevCaps( HWND hwnd, WORD wDeviceID );
VOID AmpMixSpecDevCaps( HWND hwnd, WORD wDeviceID );
VOID OvlySpecDevCaps( HWND hwnd, WORD wDeviceID );


/**************************** Global Variables. ****************************/
/* Define parameters by type */
HAB     hab;
HMQ     hmq;
HWND    hwndFrame;               /* Frame window handle */
HWND    hwndClient;              /* Client window handle-Parent of Notebook */
HWND    hwndMain;             /* Handle of window containing the Notebook */

HPOINTER hptrOriginal;           /* Handle to the default system cursor */
HPOINTER hptrWait;               /* Handle to the system wait cursor */
HPOINTER hptrCurrent;            /* Handle to the cursor in use */

BOOL    bComplete = FALSE;       /* switch for first time through */
WORD    ActualDev = 0;           /* Actual number of installed devices */

/* Define System Information (SYSINFO) Structures */
MCI_SYSINFO_PARMS      SysInfo;                                                    
MCI_SYSINFO_QUERY_NAME SysInfoName;                                                
MCI_SYSINFO_LOGDEVICE  SysInfoLogData;                                             
MCI_SYSINFO_DEVPARAMS  SysInfoDevParams;                                           

CHAR szMaxDevNameStr[MAX_DEVICE_NAME];    /* String for device name of max length */

/* Notebook page ID's */
HWND hwndDev1Info;
ULONG ulIDDev1;        

/************************************************/
/****** Structute Initialization Section ********/
/************************************************/


/*********************************************************************
*  Declare and Initialize an array DevInfo[] which will contain
*  system device information. It is of type MCI_DEVCAPS_DEVICE
*  and can contain a maximum of 20 elements (MaxDevices). The
*  DEVCAPS.H file contains the typedef information. The structure
*  elements are as follows:
*        DevInfo[].szTabID          ID for Tad Label
*        DevInfo[].hwndPageDlg      Page handle
*        DevInfo[].wDeviceType      The device type number
*        DevInfo[].szInstallName    Device Install Name
*        DevInfo[].szProductInfo    Device Product Information
*        DevInfo[].fDevSelect       Device been previously selected ?
/*********************************************************************/
MCI_DEVCAPS_DEVICE DevInfo[MaxDevices] =
{ { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 },
  { "", (HWND) 0, 0, "", "", 0 } };

 

/**********************************************************************
* Declare and Initialize an array InstalledDevice[] which will contain
* the name and number of MCI devices as defined in the MMSYSTEM.H file
* for OS/2 2.0. The array has 17 elements (MCI_MAX_SYSTEM_DEVICE_NAMES)
* which is the current definition of OS/2 2.0. The structure
* elements are as follows:
*        InstalledDevice[].szDevTypeName       Device Type Name
*        InstalledDevice[].wDeviceType         Device Type Number
/*********************************************************************/
MCI_INSTALLED_DEVICE InstalledDevice[MCI_MAX_SYSTEM_DEVICE_NAMES] =
{ { "Videotape",     1 },
  { "Videodisc",     2 },                    
  { "CDaudio",       3 },                    
  { "DAT",           4 },                    
  { "Audiotape",     5 },                    
  { "Other",         6 },                    
  { "Waveaudio",     7 },                    
  { "Sequencer",     8 },                    
  { "Ampmix",        9 },                    
  { "Overlay",       10 },
  { "Animation",     11 },                   
  { "Digitalvideo",  12 },                   
  { "Speaker",       13 },                   
  { "Headphone",     14 },                   
  { "Microphone",    15 },                   
  { "Monitor",       16 },                   
  { "CDXA",          17 }};                 
                                                        
/**********************************************************************
* Declare and Initialize an array StringCommands[] which will contain
* the list of basic srting commands that are recognized by all MCI
* devices for OS/2 2.0. The array has 13 elements (NumbStrCmds) with
* a length of MCI_STRING_LENGTH. The string commands for MCI_OPEN and
* MCI_CLOSE are handled in the procedure. The structure has the 
* following element:
*        mciStrCommands[MCI_STRING_LENGTH]   /* String Command */
/*********************************************************************/
static char szCapCommand[] = "CAPABILITY dev01 ";

#define NumbStrCmds     14

SendString StringCommand[NumbStrCmds] =
        {"can record ",         
         "has audio ",         
         "has video ",         
         "has image",
         "can eject ",
         "can play ",           
         "can save ",           
         "compound device ",
         "uses files ",         
         "can lockeject ",      
         "can setvolume ",      
         "preroll type ",       
         "preroll time ",       
         "device type " };      

/************************ End Global Variables. ****************************/
                                                                   
/****************************************************************
 *  Name:   Main
 *
 *  Description  : Window procedure for the main clent window.
 *
 *  Concepts     : None.
 *             
 *  Parameters   : None.
 *               
 *               
 *  Returns      :  Returns 0 to the operating system.
 *
 ****************************************************************/
int main( VOID )
   {
   QMSG         qmsg;
   BOOL         success;
 
   /* Initialize and get an anchor block handle */
   hab = WinInitialize( 0 );
   if ( !hab )
      {
      DosBeep( BEEP_WARN_FREQ, BEEP_WARN_DUR );
      exit( RETURN_ERROR );
      } /* endif */

   /* Create a message queue */
   hmq = WinCreateMsgQueue( hab, 0 );
   if ( !hmq)
      {
      DosBeep( BEEP_WARN_FREQ, BEEP_WARN_DUR );
      exit( RETURN_ERROR );
      } /* endif */

   /* Display the Product Infomation dialog box */
   success = WinDlgBox( HWND_DESKTOP,
                        HWND_DESKTOP,
                        ( PFNWP )ProdInfoDlgProc,
                        ( HMODULE ) 0,
                        ID_PRODINFO,
                        NULL );

   if ( !success )
      {
      DisplayMessage( "Could Not Register Class" );
      exit( RETURN_ERROR );
      } /* endif */

   /* This function handles the main window set-up */
   SetupMyMainWindow ();
   WinProcessSecondaryWindow(hwndMain);
   WinDestroySecondaryWindow(hwndMain);

   WinDestroyMsgQueue( hmq );          /* Destroy message queue */
   WinTerminate( hab );                /* Terminate application */
   return( 0 );

   }   /* End main()  */

/***********************  End of main procedure  ************************/

/*************************************************************************
 *  Name:   SetupMyMainWindow
 *
 *  Description : Procedure for setting up the Main Window.
 *
 *  Concepts : The main window for this application is a MMPM/2
 *             Secondary Window whose client is a dialog window.
 *             All that needs to be done is to use WinLoadSecondaryWindow(),
 *             specifying the resource and id for the dialog for the notebook.
 *             The system pointer is replaced with the waiting "hourglass"
 *             pointer during the window creation process.
 *
 *  Parameters   : None.
 *
 *  Returns:  Return False .
 *
 *************************************************************************/
USHORT SetupMyMainWindow()
   {
   BOOL success;

   /* Obtain handle of system pointer (cursor) */
   hptrOriginal = WinQueryPointer( HWND_DESKTOP );

   /* Obtain handle of system wait pointer (cursor) */
   hptrWait = WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE );
   hptrCurrent = hptrWait;

   /* Load the main dialog for the application notebook */
   hwndMain = WinLoadSecondaryWindow( HWND_DESKTOP,
                        HWND_DESKTOP,
                        DevDlgProc,
                        ( HMODULE ) 0,
                        ID_MYDIALOG,
                        NULL );

   WinDefaultSize(hwndMain);
   WinShowWindow(hwndMain, TRUE);
 
   return ( FALSE );

   }  /* End SetupMyMainWindow() */
/**************  End of SetupMyMainWindow procedure  ********************/

/*********************************************************************
 *  Name : ProdInfoDlgProc
 *
 *  Description : Processes all messages sent to the Product
 *                Information dialog box.
 *
 *  Concepts : Called for each message sent to the Product
 *             Information dialog box.  The Product
 *             Information box only has a button control so
 *             this routine only processes WM_COMMAND
 *             messages.  Any WM_COMMAND posted must have come
 *             from the Ok button so we dismiss the dialog
 *             upon receiving it.
 *
 * Parameters   : hwnd - Window handle to which message is addressed
 *                msg - Message type
 *                mp1 - First message parameter
 *                mp2 - Second message parameter
 *
 *  Returns : Dependent upon message sent
 *
 ****************************************************************/
MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
   {
   switch(msg)
      {
      case WM_INITDLG:
         break;

      case WM_COMMAND:
         /* No matter what the command, close the dialog */
         WinDismissDlg(hwnd, TRUE);
         break;

      default:
         return WinDefDlgProc( hwnd, msg, mp1, mp2 );
         break;
       }
   return (MRESULT)0;
   }
/********************** End of ProdInfoDlgProc *********************/
/*********************************************************************
 *  Name:   GetDevSysInfo
 *
 *  Description : This procedure queries the system for information
 *                from its MMPM2.INI file.
 *
 *  Concepts : Obtains MCI device information through SYSINFO calls.
 *             It asks for the quanity of a specific device type. If
 *             this MCI device type is present then the DevInfo[] 
 *             array is filled with the device type name, the device
 *             type number, the device install name, and the device 
 *             product information. A device counter, ActualDev, is 
 *             used to keep track of the number of actual devices
 *             found in the system. The number of devices, Actual, 
 *             is limited to be less then or equal to MaxDevices (20).
 *             
 *  Parameters   : None.
 *
 *  Returns:  Returns False if successful and True otherwise.
 *
 **********************************************************************/
BOOL GetDevSysInfo()
   {

   CHAR                   SysInfoRetBuf[128];
   DWORD                  dwRC;
   DWORD                  dwNumOfThisDevice;
   DWORD                  dwDevCount;
   int                    i, DeviceType, Max;
   CHAR                   szTempStrMaxLength[MAX_DEVICE_NAME];
   CHAR                   szDevOrdStr[5];

   szMaxDevNameStr[0] = '\0';
   szTempStrMaxLength[0] = '\0';
   Max = MCI_MAX_SYSTEM_DEVICE_NAMES;

   /************************************************************************/
   /* Iterate through the device types and determine their existence by    */
   /* their quantity. Use SYSINFO calls to query the system. The results   */
   /* will be returned in the SYSINFO structures.                          */
   /************************************************************************/
   for ( DeviceType = 0; DeviceType <= Max ; DeviceType ++ )
      {
      memset ( SysInfoRetBuf, 0x00, 128) ;      /* Initialize structures */
      memset ( &SysInfo, 0x00, sizeof (MCI_SYSINFO_PARMS)) ;
      SysInfo.wDeviceType =                /* Current device type */
            InstalledDevice[DeviceType].wDeviceType;
      SysInfo.lpstrReturn = (LPSTR)&SysInfoRetBuf;    /* Return buffer */
      SysInfo.dwRetSize   = 128;

      /* Ask for quantity of this device */ 
      dwRC = mciSendCommand (0, MCI_SYSINFO,      
                          MCI_SYSINFO_QUANTITY | MCI_WAIT,
                          (DWORD) &SysInfo, 0);

      if (LOUSHORT(dwRC) == MCIERR_SUCCESS)
         {
         dwNumOfThisDevice = SysInfoRetBuf[0] - '0';

         /************************************************************/
         /* If more than one device of this type, handle all of them */
         /************************************************************/
         for (dwDevCount=1; dwDevCount <= dwNumOfThisDevice; dwDevCount++)
            {
            /****************************************************/
            /* Get the INSTALL NAME of this device              */
            /****************************************************/
            SysInfo.wDeviceType = InstalledDevice[DeviceType].wDeviceType;
            SysInfo.dwNumber = dwDevCount;
            memset ( SysInfoRetBuf, 0x00, 128) ;
            SysInfo.dwItem  = MCI_SYSINFO_QUERY_NAMES ; /* Set Command Item */
            SysInfo.pSysInfoParm = (PVOID)&SysInfoName;

            memset ( &SysInfoName, 0x00, sizeof(MCI_SYSINFO_QUERY_NAME) ) ;
            SysInfoName.usDeviceType = InstalledDevice[DeviceType].wDeviceType;
            SysInfoName.usDeviceOrd = dwDevCount;

            /* Query device install name */ 
            dwRC = mciSendCommand (0, MCI_SYSINFO,   
                           MCI_SYSINFO_ITEM | MCI_WAIT,
                           (DWORD) &SysInfo, 0);

            if (LOUSHORT(dwRC) == MCIERR_SUCCESS)
               {
               /*************************************************/
               /* Get the PRODUCT INFORMATION on this device.   */
               /*************************************************/
               SysInfo.wDeviceType = InstalledDevice[DeviceType].wDeviceType;
               SysInfo.dwNumber = dwDevCount;
               memset ( SysInfoRetBuf, 0x00, 128) ;
               SysInfo.dwItem      = MCI_SYSINFO_QUERY_DRIVER;  
               SysInfo.pSysInfoParm = (PVOID)&SysInfoLogData;

               memset ( &SysInfoLogData, 0x00, sizeof(MCI_SYSINFO_LOGDEVICE)) ;
               strcpy (SysInfoLogData.szInstallName, SysInfoName.szInstallName);

               /* Query driver for product information */ 
               dwRC = mciSendCommand (0, MCI_SYSINFO,     
                              MCI_SYSINFO_ITEM | MCI_WAIT,
                              (DWORD) &SysInfo, 0);

               /*  SysInfoLogData.szProductInfo will contain the product name */
               /* If the found type is equal to original type, copy info  */
               if ( SysInfo.wDeviceType == InstalledDevice[DeviceType].wDeviceType )
                  {
                  strcpy( DevInfo[ActualDev].szTabID,    /* Copy Device Type Name */
                     InstalledDevice[DeviceType].szDevTypeName );

                  /* Append on the ordinal value for this instance of the device type */
                  if (dwDevCount > 1)
                     {
                     memset (szDevOrdStr, 0x00, 5);
                     szDevOrdStr[0] = '0';
                     szDevOrdStr[1] = '0' + dwDevCount;
                     strcat(DevInfo[ActualDev].szTabID, szDevOrdStr);
                     }

                  DevInfo[ActualDev].wDeviceType =       /* Copy device Type Number */
                     InstalledDevice[DeviceType].wDeviceType ;

                  strcpy( DevInfo[ActualDev].szProductInfo, /* Copy Device Product Information */
                     SysInfoLogData.szProductInfo );

                  strcpy( DevInfo[ActualDev].szInstallName, /* Copy device Install Name */
                     SysInfoLogData.szInstallName );

                  if ( strlen(DevInfo[ActualDev].szInstallName) >
                        strlen( szTempStrMaxLength ))    /* Compare for longest install name */
                     strcpy(szTempStrMaxLength, DevInfo[ActualDev].szInstallName);

                  ActualDev ++;                    /* Increment Acual devices installed */
                  if ( ActualDev > MaxDevices )    /* Check for device limit */
                     {
                     ActualDev = MaxDevices;       /* If more than 20 devices Then */
                     break;                        /* only allow for 20.           */
                     } /* end-if */

                  } /* endif */

               } /* endif dwRC #2 */
            }    /* endfor dwDevCount */

         } /* endif dwRC #1 */

      if ( ActualDev >= MaxDevices )          /* Allow maximum of 20 devices */
         break;

      } /* endfor DeviceType */
   if ( ActualDev == 0)               /* If no devices were found return TRUE */
      return (TRUE);
   strcpy( szMaxDevNameStr, szTempStrMaxLength );  /* Copy longest Install name */
   return (FALSE);                                 /* and use for Tab Length    */
   }

/********************** End of GetDevSysInfo ***********************/
/****************************************************************************
 *  Name:   DevDlgProc
 *
 *  Description : Main dialog procedure for the notebook application.
 *
 *  Concepts : Upon receiving WM_INITDLG this procedure calls 
 *             GetDevSysInfo() to obtain device information for
 *             the notebook and calls SetupMyDevInfoPages() to 
 *             create the notebook and display device information.
 *             This procedure also traps WM_CONTROL messages for 
 *             the notebook and processes them.
 *             
 * Parameters   : hwndDlg - Window handle to which message is addressed
 *                msg - Message type
 *                mp1 - First message parameter
 *                mp2 - Second message parameter
 *
 *  Returns:  Return values are determined by each message
 *
 ****************************************************************************/
MRESULT EXPENTRY DevDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   HWND hwndNotebook;
   USHORT usPageStyle;
   USHORT usPageOrder;

   switch ( msg )
      {
      case WM_INITDLG:
         {
         SWP swp;
         /* Set The system wait pointer because this takes a little time */
         WinSetPointer( HWND_DESKTOP, hptrWait );
         if (GetDevSysInfo() )  /* Get device information from the system */
            {                   /* If SYSINFO call failed the exit */
            DisplayMessage("Could not get System information");
            exit(RETURN_ERROR);
            }

         /* Set-up notebook pages for each device found */ 
         SetupMyDevInfoPages (hwndDlg); 

         /* Add "Default Size" to System Menu */
         WinInsertDefaultSize(hwndDlg, "Default size");

         /* Restore original system pointer */ 
         WinSetPointer( HWND_DESKTOP, hptrOriginal );  

         /* Make sure that notebook can fit into dialog window */
         WinQueryWindowPos(WinWindowFromID(hwndDlg, ID_DEVICE_NOTEBOOK),
            &swp);
         WinSetWindowPos(hwndDlg, 0, 0, 0, 2 * swp.x + swp.cx, 2 * swp.y + swp.cy,
            SWP_SIZE);
         }
         break;

      case WM_COMMAND:                    /* Posted by pushbutton or key  */
         /* PM sends a WM_COMMAND message when the user presses either
          * the Enter or Escape pushbuttons.  */
         switch( SHORT1FROMMP( mp1 ) )     /* Extract the command value    */
            {
            case DID_OK:        /* The Enter pushbutton or key. */

            case DID_CANCEL:    /* The Cancel pushbutton or Escape key */
               WinDismissSecondaryWindow( hwndDlg, TRUE );  /* Removes the dialog box  */
               break;

            default:
               break;
            }
         break;
      case WM_HELP:           /*  Display Help dialog when F1 is pressed */
         WinDlgBox(HWND_DESKTOP, hwndDlg,
               DevHelpInfoDlgProc,
               (HMODULE) 0, ID_HELP_INFO_DLG,
               0);
         break;

      case WM_CONTROL:
         switch( SHORT1FROMMP( mp1 ) )      /* Extract the control ID   */
            {
            case ID_DEVICE_NOTEBOOK:         /* Notify data in mp2 */
               /* What kind of NOTEBOOK notification is it? */
               switch( SHORT2FROMMP( mp1 ) ) /* Notify msg type in short2 */
                  {
                  case BKN_PAGESELECTED:     /* Process the selected device */
                     HandleInfoPageSelect ((PAGESELECTNOTIFY *)mp2);
                     break;
                  }
               break;
            } /* endswitch */
         break;

      default:
         /* Any event messages that the dialog procedure has not processed
          * come here and are processed by WinDefDlgProc. This call
          * MUST exist in your dialog procedure. */
         return((MRESULT)WinDefSecondaryWindowProc(hwndDlg, msg, mp1, mp2));
      }
   return((MRESULT)WinDefSecondaryWindowProc(hwndDlg, msg, mp1, mp2));
   }

/********************** End of DevDlgProc  ************************/
/*******************************************************************
 *  Name:   HandleInfoPageSelect
 *
 *  Description : Determine which specific page was selected, hide
 *                the old one (if any), show the new one, and update
 *                any information that needs to be displayed in the
 *                new one.
 *
 *  Concepts : Query the handle of the current notebook page and
 *             save it. Query the handle of the new selected 
 *             notebook page. Call ProcDevCaps for the device whose
 *             name is located on the major tab of this page. This 
 *             is the device that is to be queried for device
 *             capabilities.
 *             
 *  Parameters   : None.
 *
 *  Returns:  Return False.
 *
 **********************************************************************/
HandleInfoPageSelect (PAGESELECTNOTIFY *PageSelectStruct)
   {
   HWND hwndNewPage, hwndCurPage;   /* handle for current and new pages */
   USHORT usDevice;         /* Some identifier for the device selected */
   int i = 0;

   /* Query the current page handle */
   hwndCurPage = (HWND)WinSendMsg(PageSelectStruct->hwndBook,
                    BKM_QUERYPAGEWINDOWHWND,
                    MPFROMLONG (PageSelectStruct->ulPageIdCur),
                    (MPARAM)0);

   /* Query the selected page handle */
   hwndNewPage = (HWND)WinSendMsg(PageSelectStruct->hwndBook,
                    BKM_QUERYPAGEWINDOWHWND,
                    MPFROMLONG (PageSelectStruct->ulPageIdNew),
                    (MPARAM)0);

   if (hwndNewPage)           /* If valid handle then process it */
      {
      WinSetPointer( HWND_DESKTOP, hptrWait );  /* Set system wait pointer */
      ProcessDevCaps( hwndNewPage );            /* Query device capabilities */

      /* Restore original system pointer */ 
      WinSetPointer( HWND_DESKTOP, hptrOriginal );  
      }

   return (FALSE) ;
   }  /* End HandleInfoPageselect */

/******************** End of HandleInfoPageselect  *********************/
/*********************************************************************
 *  Name : SetupMyDevInfoPages
 *
 *  Description : Sets up the notebook pages and tab labels
 *
 *  Concepts : This procedure is called when WM_INITDLG is sent to 
 *             the notebook. It calculates the size of the major 
 *             tabs based on the device with the longest install 
 *             name. It creates and inserts a page for each device
 *             that exists in DevInfo[]. A major tab is created for 
 *             each page with the install name for that device as 
 *             its label and then the page dialog is loaded. The 
 *             product information is inserted on the status text 
 *             line of the notebook. The procedure makes an 
 *             association between the application page window 
 *             handle and the created page.
 *
 *  Parameters :  hwnd - Handle of the Notebook dialog
 *
 *  Returns: FALSE if The NoteBook is initialized successfully, 
 *           TRUE otherwise.
 *
 ****************************************************************/
BOOL SetupMyDevInfoPages( HWND hwnd )
   {
   HWND hwndNotebook;
   ULONG ulRetCode;
   USHORT usTabHeight, usTabWidth;     /* Tab height and length variables */
   USHORT usFontW, usFontH ;           /* Font height and width variables */
   USHORT usMaxTabCharsH, usMaxTabCharsW ;  /* max tab height and width */
   int i, j;

   /* Set Tab Size and Font Dimensions */
   usFontW = usFontH = 10;
   usMaxTabCharsW = 10;
   usMaxTabCharsH = 1;
 
   usTabHeight = usFontH * usMaxTabCharsH + 14;
   usTabWidth = (USHORT)strlen(szMaxDevNameStr) * usMaxTabCharsW + 16;

   /* Send notebook set-dimensions command for tabs */
   ulRetCode = ( LONG ) WinSendDlgItemMsg( hwnd,
                                 ID_DEVICE_NOTEBOOK,
                                 BKM_SETDIMENSIONS,
                                 MPFROM2SHORT( usTabWidth, usTabHeight ),
                                 MPFROMLONG( ( LONG ) BKA_MAJORTAB ) );
   if ( !ulRetCode )
      return (TRUE);

   /* Get Handle of the notebook dialog */
   hwndNotebook = WinWindowFromID( hwnd, ID_DEVICE_NOTEBOOK );

   /* create pages for all Actual found devices */
   for ( i = 0; i < ActualDev; i++ )
      {
      /* Create and Insert info page for this device  */
      ulIDDev1 = ( LONG ) WinSendDlgItemMsg( hwnd,
                         ID_DEVICE_NOTEBOOK,
                         BKM_INSERTPAGE,
                         NULL,
                         MPFROM2SHORT( ( BKA_AUTOPAGESIZE | 
                           BKA_STATUSTEXTON | BKA_MAJOR ), BKA_LAST ) );

      if ( !ulIDDev1 )
         return TRUE;

      /* Set the tab text( NSTALL NAME) for this device info page */
      if ( !WinSendDlgItemMsg( hwnd,
                          ID_DEVICE_NOTEBOOK,
                          BKM_SETTABTEXT,
                          MPFROMLONG( ulIDDev1 ),
                          MPFROMP( DevInfo[i].szInstallName ) ) )
         return TRUE;

      /* Set up the dialog for this info page */
      DevInfo[i].hwndPageDlg =  WinLoadDlg( hwndNotebook,
                                         hwndNotebook,
                                         Dev1InfoPageDlgProc,
                                         ( HMODULE ) 0,
                                         ID_DEVICEDIALOG,
                                         NULL );
 
      if ( !DevInfo[i].hwndPageDlg )
         return (TRUE);
      /* Display the Product Information for this device on the 
         notebook status text line for this page */
      if ( !WinSendDlgItemMsg( hwnd,            
                               ID_DEVICE_NOTEBOOK,
                               BKM_SETSTATUSLINETEXT,
                               MPFROMLONG( ulIDDev1 ),
                               MPFROMP( DevInfo[i].szProductInfo ) ) )
         return TRUE;

      /* Associate the application page hwnd with this notebook page */
      if ( !WinSendDlgItemMsg( hwnd,
                               ID_DEVICE_NOTEBOOK,
                               BKM_SETPAGEWINDOWHWND,
                               MPFROMLONG( ulIDDev1 ),
                               MPFROMHWND( DevInfo[i].hwndPageDlg ) ) )
         return TRUE;
      } /* endfor */

   return FALSE;
   }           /* End of InitializeNotebook    */

/******************** End of SetupMyDevInfoPages  ************************/
/**********************************************************************
 *  Name:   ProcessDevCaps
 *
 *  Description : This procedure performs the device capabilities
 *                processing for the selected MCI device.
 *
 *  Concepts : Finds the device with the given page handle (hwnd).
 *             Opens the specified device with mciSendString. If the
 *             device cannot be opened then the error is reported to
 *             the user. The errors fall into the following types:
 *             1) Hardware error. The device hardware may not be
 *                present or usable.
 *             2) MMPM/2 cannot load device driver. The device may
 *                not have or need a driver,...ie SPEAKERS.
 *             After opening the device the list of string commands
 *             are issued with mcisendString for the Command String 
 *             Interface. If the current MCI device is of type
 *             videodisc or overlay then the appropriate function is
 *             called to query the device for its device specific 
 *             capabilities via the Command Message Interface 
 *             (mciSendCommand). This procedure displays the command 
 *             and its result on the notebook page for each device 
 *             selected. Once a device has been queried, a flag is 
 *             set to display the device's data upon additional 
 *             selections of this device instead of re-quering the 
 *             device each time it is selected. The procedure then 
 *             closes the device with the mciSenString command.
 *
 *  Parameters   : hwnd - Window handle to which message is addressed
 *
 *  Returns:  Return FALSE if successfull and TRUE otherwise.
 *
 **********************************************************************/
VOID ProcessDevCaps( HWND hwnd )
   {
 
   DWORD dwRC;


   CHAR szOpenBuffer[128];       /* Open string buffer */
   CHAR szMCIReturnString[MCI_RETURN_STRING_LENGTH];   /* Return string buffer */
   CHAR szErrorString[ MCI_ERROR_STRING_LENGTH ]; /* Error string buffer */
   ULONG ulSendStringRC = 0,        /* Send string return code string */
         ulGetErrorStringRC = 0;    /* Get error string variable */
   ULONG ulSendCommandRC = 0;       /* Send command return code string */
   int  i = 0;
   CHAR szTabTempID[MAX_DEVICE_NAME];  /* Temp string for Tab */
   HWND hwndTempPageDlg;               /* Temp hwnd for page  */
   WORD wDeviceID, wDevTempType;
   CHAR szTempStr[100];                /* Temp string for output */

   /* find the device with this page hwnd */
   for ( i=0 ; i<ActualDev ; i++ )
      {
      if ( DevInfo[i].hwndPageDlg == hwnd )
         {
         if ( DevInfo[i].fDevSelect )   /* If the device has been previously */
            return;                     /* selected then return and show the */
         else                           /* page with the information */
         {
         strcpy( szTabTempID, DevInfo[i].szTabID );   /* copy device tabID */
         hwndTempPageDlg = DevInfo[i].hwndPageDlg;    /* copy device page hwnd */
         wDevTempType = DevInfo[i].wDeviceType;       /* copy device type */
         DevInfo[i].fDevSelect = 1;                   /* set select to true */
         WinSetDlgItemText( hwndTempPageDlg, ID_DEVICETYPE,    /* display device */
                                szTabTempID);                  /* on notebook page */
         } /* endif */
      break;
      } /* endif */
   } /* endfor */

   /* construct open string for this device */
   strcpy( szOpenBuffer, "open ");
   strcat( szOpenBuffer, szTabTempID);    
   strcat( szOpenBuffer, " alias dev01 wait ");
                                   
   /* open the device */
   ulSendStringRC = mciSendString( (LPSTR)szOpenBuffer,
                       (LPSTR)szMCIReturnString,
                        (WORD) MCI_RETURN_STRING_LENGTH,
                        0,
                        0 );


   /* If the device cannot be opened, inform the user, return and  */
   /* display the error */
   if (ulSendStringRC)
      {
      ulGetErrorStringRC = mciGetErrorString( ulSendStringRC,
                                             (LPSTR)szTempStr, (WORD)100);

      DisplayMessage (szTempStr);
      /* Send the error string to the list box */
      WinSendDlgItemMsg ( hwndTempPageDlg,
                          ID_CAPS_LISTBOX,                       
                          LM_INSERTITEM,                         
                          MPFROMSHORT(LIT_END),                  
                          MPFROMP (szTempStr));
      }
   else              /* the open was successful  */
      {
      /* Query the device with the list of string commands */
      for ( i=0 ; i<NumbStrCmds ; i++ )
         {
         strcpy( szTempStr, szCapCommand);      /* build the command string */
         strcat( szTempStr, StringCommand[i].mciStrCommands);
         strcat( szTempStr, "wait");            /* add wait to the command */
         /* send the srting command */
         ulSendStringRC = mciSendString( (LPSTR)szTempStr,
                                (LPSTR)szMCIReturnString,
                                (WORD)MCI_RETURN_STRING_LENGTH,
                                0,
                                0 );

         /* Build output string */
         strcpy (szTempStr, StringCommand[i].mciStrCommands);
         strcat (szTempStr, "     ");
         strcat (szTempStr, szMCIReturnString);

         /* Send the string command and the result to the list box */
         WinSendDlgItemMsg( hwndTempPageDlg,
                            ID_CAPS_LISTBOX,                       
                            LM_INSERTITEM,                         
                            MPFROMSHORT(LIT_END),                  
                            MPFROMP (szTempStr));

         } /* endfor */

      /**************************/
      /* Attempt mciSendCommand */
      /**************************/

      /* Get the device ID from its alias for use with the mciSendCommand */
      wDeviceID = mciGetDeviceID((LPSTR) "dev01");

      /* if the device is of type videodisc or overlay then process */
      /* the device specific capabilities for these device types */
      if ( wDevTempType == 2 )      /* type videodisc */
         VdiscSpecDevCaps( hwndTempPageDlg, wDeviceID );
      else if ( wDevTempType == 10 )      /* type overlay */
         OvlySpecDevCaps( hwndTempPageDlg, wDeviceID );

      /* Close the Device dev01 */
      ulSendStringRC = mciSendString( (LPSTR)"close dev01",
                               (LPSTR)szMCIReturnString,
                               (WORD)MCI_RETURN_STRING_LENGTH,
                               0,
                               0 );

      } /* end if */

   } /* End ProcessDevCaps */
 
/******************** End of ProcessDevCaps ************************/
/*********************************************************************
 *  Name:   VdiscSpecDevCaps
 *
 *  Description : This procedure processes the device specific
 *                capabilities for videodisc devices.
 *
 *  Concepts : The procedure sets up an array and iterates through
 *             the defined list of specific capabilities for videodisc
 *             devices. This list is defined in the DEVCAPS.H file.
 *             The capabilities are quieried for both CAV and CLV 
 *             videodisc types. The command and the results are 
 *             displayed on the page for the selected device.
 *             
 *  Parameters   : hwnd - Page handle for this device
 *                wDevice - the device type number
 *
 *  Returns:  Return Void.
 *
 *********************************************************************/
VOID VdiscSpecDevCaps( HWND hwnd, WORD wDeviceID )
   {
   MCI_GETDEVCAPS_PARMS    GetDevCapParms; /* DEVCAPS parm structure */      
   char szCapsNumReturn[10];                                                 
   WORD mciMessage;                        /* structure message */           
   DWORD mciFlags;                         /* structure message flags */     
   CHAR szTempStr[100];                    /* temp string for output */      
   CHAR szErrorString[ MCI_ERROR_STRING_LENGTH ]; /* error string buffer */

   DWORD dwRC;
   ULONG ulGetErrorStringRC = 0;
   ULONG ulSendCommandRC = 0;
   WORD wVdisctype;
   DWORD dwCommand;


   wVdisctype = 0;                           /* loop variables */
   dwCommand = 0;

   mciMessage = MCI_GETDEVCAPS;            /* Set the message field */
   memset (&GetDevCapParms, 0x00, sizeof (GetDevCapParms)); 

   /* Send all commands for both CAV and CLV videodiscs */
   for ( wVdisctype=0 ; wVdisctype<2 ; wVdisctype++ )   
      {
      mciFlags = MCI_WAIT | MCI_GETDEVCAPS_ITEM;  /* set message flags */

      /* Iterate through the videodisc specific capabilities */
      for ( dwCommand=VdiscCmds_Begin ; 
            dwCommand<=VdiscCmds_End ; 
            dwCommand++ )
         {
         GetDevCapParms.dwItem = dwCommand;    /* set the command item */

         if ( wVdisctype == 0 )     /* FIRST pass is CAV */
            {
            mciFlags = mciFlags | MCI_VD_GETDEVCAPS_CAV;
            strcpy( szTempStr, "CAV ");
            } 
         else
            {
            mciFlags = mciFlags | MCI_VD_GETDEVCAPS_CLV;
            strcpy( szTempStr, "CLV ");
            } /* end if */

         ulSendCommandRC = mciSendCommand( wDeviceID,
                                 mciMessage,
                                 mciFlags,
                                 (DWORD) &GetDevCapParms,
                                 0 );

         /*********************************************/
         /* If successful, build the "answer"  string */
         /*********************************************/
         if (LOUSHORT(ulSendCommandRC) == MCIERR_SUCCESS )
            {
            switch ( GetDevCapParms.dwItem )
               {
               case MCI_VD_GETDEVCAPS_CAN_REVERSE:
                  strcat(szTempStr, "can reverse ");
                  if ( GetDevCapParms.dwReturn)
                     strcat( szTempStr, " TRUE");
                  else
                     strcat( szTempStr, " FALSE");
                  break;

               case MCI_VD_GETDEVCAPS_FAST_RATE:
                  strcat(szTempStr, "fast play rate ");
                  if ( GetDevCapParms.dwReturn)
                     strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                              szCapsNumReturn, 10));
                  else
                     strcat( szTempStr, " FALSE");
                  break;

               case MCI_VD_GETDEVCAPS_SLOW_RATE:
                  strcat(szTempStr, "slow play rate ");
                  if ( GetDevCapParms.dwReturn)
                     strcat (szTempStr, _itoa (GetDevCapParms.dwReturn, 
                                               szCapsNumReturn, 10));
                  else
                     strcat( szTempStr, " FALSE");
                  break;

               case MCI_VD_GETDEVCAPS_NORMAL_RATE:
                  strcat(szTempStr, "normal rate ");
                  if ( GetDevCapParms.dwReturn)
                     strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                               szCapsNumReturn, 10));
                  else
                     strcat( szTempStr, " FALSE");
                  break;
           
               case MCI_VD_GETDEVCAPS_MAXIMUM_RATE:
                  strcat(szTempStr, "maximum rate ");
                  if ( GetDevCapParms.dwReturn)
                     strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                               szCapsNumReturn, 10));
                  else
                     strcat( szTempStr, " FALSE");
                  break;
           
               case MCI_VD_GETDEVCAPS_MINIMUM_RATE:
                  strcat(szTempStr, "minimum rate ");
                  if ( GetDevCapParms.dwReturn)
                     strcat (szTempStr, _itoa (GetDevCapParms.dwReturn, 
                                               szCapsNumReturn, 10));
                  else 
                     strcat( szTempStr, " FALSE");
                  break;
                                                       
               default:
                  break;

               } /* end Switch */

            } /* endif */
         else                 /* get the error string */
            {
            szTempStr[0] = '\0';
            mciGetErrorString( ulSendCommandRC,
                          (LPSTR) szErrorString,
                          (WORD) MCI_ERROR_STRING_LENGTH );
            DisplayMessage( szErrorString );
            strcpy(szTempStr, szErrorString);
            } /* endif else */


         /* Send the command message and the result to the list box */
          WinSendDlgItemMsg (hwnd,
                             ID_CAPS_LISTBOX,
                             LM_INSERTITEM,
                             MPFROMSHORT(LIT_END),
                             MPFROMP (szTempStr));

         } /* endfor j inner loop */

      } /* endfor i outer loop */
   }  /* end Vdisc.... */

/************************* End VdiscSpecDevCaps ***********************/
/**********************************************************************
 *  Name:   OvlySpecDevcaps
 *
 *  Description : This procedure processes the device specific
 *                capabilities for overlay devices.
 *
 *  Concepts : The procedure sets up an array and iterates through
 *             the defined list of specific capabilities for overlay
 *             devices. This list is defined in the DEVCAPS.H file.
 *             The command and the results are displayed on the page
 *             for the selected device.
 *             
 *  Parameters   : hwnd - Page handle for this device
 *                wDevice - the device type number
 *
 *  Returns:  Return Void.
 *
 *********************************************************************/
VOID OvlySpecDevCaps( HWND hwnd, WORD wDeviceID )
   {
   MCI_GETDEVCAPS_PARMS    GetDevCapParms;      /* DEVCAPS parm structure */
   char szCapsNumReturn[10];
   WORD mciMessage;                             /* structure message */
   DWORD mciFlags;                              /* structure message flags */
   CHAR szTempStr[100];                         /* temp string for output */
   CHAR szErrorString[ MCI_ERROR_STRING_LENGTH ]; /* error string buffer */
   DWORD dwRC;
   ULONG ulGetErrorStringRC = 0;
   ULONG ulSendCommandRC = 0;
   DWORD dwCommand;                                  /* loop variable */

   dwCommand = 0;

   mciMessage = MCI_GETDEVCAPS;            /* Set the message field */
   mciFlags = MCI_WAIT | MCI_GETDEVCAPS_ITEM;  /* set the message flags */
   memset (&GetDevCapParms, 0x00, sizeof (GetDevCapParms));  

   /********************************************************************/
   /* Iterate through the overlay specific device capabilities as      */
   /* defined in the DEVCAPS.H file                                    */
   /********************************************************************/
   for ( dwCommand=OvlyCmds_Begin;
         dwCommand<=OvlyCmds_End;
         dwCommand++ )
      {
      GetDevCapParms.dwItem = dwCommand;    /* set the command item */

      /* send the command message */
      ulSendCommandRC = mciSendCommand( wDeviceID,
                                       mciMessage,
                                       mciFlags,
                                       (DWORD) &GetDevCapParms,
                                       0 );

      if (LOUSHORT(ulSendCommandRC) == MCIERR_SUCCESS )
         {
         switch ( GetDevCapParms.dwItem )
            {
            /****************************************************************/
            /* switch on the command item and build the appropriate string  */
            /* MCI_GETDEVCAPS item values for Video Overlay                 */
            /****************************************************************/
            case MCI_OVLY_GETDEVCAPS_CAN_STRETCH:
               strcpy(szTempStr, "can stretch ");
               if ( GetDevCapParms.dwReturn)
                  strcat( szTempStr, " TRUE");
               else 
                  strcat( szTempStr, " FALSE");
               break;

            case MCI_OVLY_GETDEVCAPS_CAN_DISTORT:
               strcpy(szTempStr, "can distort ");
               if ( GetDevCapParms.dwReturn)
                  strcat( szTempStr, " TRUE");
               else 
                  strcat( szTempStr, " FALSE");
               break;

            case MCI_OVLY_GETDEVCAPS_VIDEO_X_EXTENT:
               strcpy(szTempStr, "video x extent ");
               if ( GetDevCapParms.dwReturn)
                  strcat (szTempStr, _itoa (GetDevCapParms.dwReturn, 
                                            szCapsNumReturn, 10));
               else 
                  strcat( szTempStr, " FALSE");
               break;
        
            case MCI_OVLY_GETDEVCAPS_VIDEO_Y_EXTENT:
               strcpy(szTempStr, "video y extent ");
               if ( GetDevCapParms.dwReturn)
                  strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                            szCapsNumReturn, 10));
               else 
                  strcat( szTempStr, " FALSE");
               break;

            case MCI_OVLY_GETDEVCAPS_IMAGE_X_EXTENT:
               strcpy(szTempStr, "image x extent ");
               if ( GetDevCapParms.dwReturn)
                  strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                            szCapsNumReturn, 10));
               else 
                  strcat( szTempStr, " FALSE");
               break;
        
            case MCI_OVLY_GETDEVCAPS_IMAGE_Y_EXTENT:
               strcpy(szTempStr, "image y extent ");
               if ( GetDevCapParms.dwReturn)
                  strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                            szCapsNumReturn, 10));
               else 
                  strcat( szTempStr, " FALSE");
               break;

            case MCI_OVLY_GETDEVCAPS_MAX_WINDOWS:
               strcpy(szTempStr, "max windows ");
               if ( GetDevCapParms.dwReturn)
                  strcat (szTempStr, _itoa (GetDevCapParms.dwReturn,
                                            szCapsNumReturn, 10));
               else 
                  strcat( szTempStr, " FALSE");
               break;
        
            case MCI_OVLY_GETDEVCAPS_CAN_FREEZE:
               strcpy(szTempStr, "can freeze ");
               if ( GetDevCapParms.dwReturn)
                  strcat( szTempStr, " TRUE");
               else 
                  strcat( szTempStr, " FALSE");
               break;

            case MCI_OVLY_GETDEVCAPS_OVERLAY_GRAPHICS:
               strcpy(szTempStr, "can overlay graphics ");
               if ( GetDevCapParms.dwReturn)
                  strcat( szTempStr, " TRUE");
               else 
                  strcat( szTempStr, " FALSE");
               break;
        
            default:
               break;
        
            } /* end Switch */
        
         } /* endif */
      else              /* get the error string */
         {
         szTempStr[0] = '\0';
         mciGetErrorString( ulSendCommandRC,
                          (LPSTR) szErrorString,
                          (WORD) MCI_ERROR_STRING_LENGTH );
         DisplayMessage( szErrorString );
         strcpy(szTempStr, szErrorString);
         } /* endif else */

      /* Send the command message and the result to the list box */
      WinSendDlgItemMsg (hwnd,
                         ID_CAPS_LISTBOX,
                         LM_INSERTITEM,
                         MPFROMSHORT(LIT_END),
                         MPFROMP (szTempStr));

      } /* endfor j loop */

   }  /* end Ovly.... */

/********************* End OvlySpecDevCaps ***************************/
/****************************************************************
 *  Name:   Dev1InfoPageDlgProc
 *
 *  Description : Window procedure for the main clent window.
 *
 *  Concepts : Processes the messages sent to the main client
 *             window.  This routine processes the basic
 *             messages all client windows should process and
 *             passes all others onto UserWndProc where the
 *             developer can process any others.
 *
 * Parameters   : hwnd - Window handle to which message is addressed
 *                msg - Message type
 *                mp1 - First message parameter
 *                mp2 - Second message parameter
 *
 *  Returns:  Return values are determined by each message
 *
 ****************************************************************/
MRESULT EXPENTRY Dev1InfoPageDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   USHORT usDevice;
   switch ( msg )
      {
      case WM_INITDLG:
         break;

      case WM_HELP:              /* Display Help dialog when F1 is pressed */
         WinDlgBox(HWND_DESKTOP, hwndDlg,
                  DevHelpInfoDlgProc,
                  (HMODULE) 0, ID_HELP_INFO_DLG,
                  0);
         break;

      case WM_COMMAND:                    /* Posted by pushbutton or key  */
         switch (COMMANDMSG (&msg) -> cmd)
            {
            /* Display the Help dialog box when the Help button is pressed */
            case ID_HELP_INFO_DLG:

               WinDlgBox(HWND_DESKTOP, hwndDlg,
                         DevHelpInfoDlgProc,
                         (HMODULE) 0, ID_HELP_INFO_DLG,
                         0);
               break;
            }
         break;

      default:
         /******************************************************************/
         /* Any event messages that the dialog procedure has not processed */
         /* come here and are processed by WinDefDlgProc.                  */
         /* This call MUST exist in your dialog procedure.                 */
         /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }
/********************** End of DevInfoPageDlgProc ************************/
/****************************************************************
 *  Name:   DevHelpInfoDlgProc 
 *
 *  Description : This dialog procedure handles the dialog help
 *                window for DEVCAPS. It displays a window with 
 *                a list box where the help information text is 
 *                displayed. The list box is scrollable. The dialog
 *                box has a system menu only. Thus, the user must 
 *                close it from the system menu.
 *
 *  Concepts : List Box with help information.
 *
 *  Parameters   : hwnd - Window handle to which message is addressed
 *                 msg - Message type
 *                 mp1 - First message parameter
 *                 mp2 - Second message parameter
 *
 *  Returns:  Return values are determined by each message
 *
 ****************************************************************/
MRESULT EXPENTRY DevHelpInfoDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   int count;
   CHAR szTempStr[100];
   LONG lStringLength;

   switch ( msg )
      {
      case WM_INITDLG:
         /* Create help dialog and insert the lines of text */
         for ( count=0 ; count<NUM_HELP_LINES ; count++ )
            {
            lStringLength = WinLoadString( hab,
                            (HMODULE) 0,
                            (ULONG)ID_HELP_TEXT_LINE_1 + count,
                            (LONG)100,
                            szTempStr);

            WinSendDlgItemMsg( hwndDlg,
                              ID_HELP_LISTBOX,
                              LM_INSERTITEM,                         
                              MPFROMSHORT(LIT_END),                  
                              MPFROMP (szTempStr));
            } /* endfor */
         break;
      default:
          /******************************************************************/
          /* Any event messages that the dialog procedure has not processed */
          /* come here and are processed by WinDefDlgProc.                  */
          /* This call MUST exist in your dialog procedure.                 */
          /******************************************************************/
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }

/******************** End of DevHelpUnfoDlgProc **********************/
/****************************************************************
 *  Name:   DisplayMessage
 *
 *  Description : Displays error messages.
 *
 *  Concepts : Displays a message box containing an error
 *             message each time called.
 *
 *  Parameters   : str - error message to display
 *
 *  Returns:  Void
 *
 ****************************************************************/
VOID DisplayMessage(PCH str)
   {
   /********************************************************************/
   /* This routine calls WinMessageBox to dsplay error messages to the */
   /* user.                                                            */
   /********************************************************************/

   WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) str,
      (PCH) "Error MessageBox For the DEVCAPS Sample", ID_MESSAGEBOX,
      MB_OK | MB_APPLMODAL | MB_MOVEABLE );
   }
/******************* End of DisplayMessage procedure ********************/


