//*****************************************************************************
//
// File Name: Vidview.c
//
// ****!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*****
// ****************************************************************************
// *+*+* NOTE: This file contains both '//' style comments and "/*  */"   *+*+*
// **+**       style comments.  You will have to set the the /Ss+ compile *****
// *+*+*       option for this file to be compiled.                       *+*+*
// ****************************************************************************
// ****!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*****
//
// Description : VIDVIEW sample application source file.
//
// Copyright (C) 1992 IBM Corporation
//
//     DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
//     sample code created by IBM Corporation. This sample code is not
//     part of any standard or IBM product and is provided to you solely
//     for  the purpose of assisting you in the development of your
//     applications.  The code is provided "AS IS", without
//     warranty of any kind.  IBM shall not be liable for any damages
//     arising out of your use of the sample code, even if they have been
//     advised of the possibility of such damages.                                                    *
//
//
// A. This sample application currently supports both NTSC and PAL input.
//
// B. Using this sample application.
//
//    1) This app initially displays a frame window with a
//       sizing border and a 344 by 240 client area (if NTSC input)
//       in which video is displayed.  The aspect ratio of the client
//       area is 43:30 (344:240).  The app also sets a video source
//       rectangle size of 688 by 480, which is also a 43:30 aspect
//       ratio (688:480).  Since source and destination rectangles
//       have the same aspect ratio, the video appears properly
//       proportioned, not stretched or squashed.
//
//       If the input is PAL, this app initially displays a frame
//       window of 344 by 274, source 688 by 548, aspect ratio 172:137).
//
//       The user can drag-and-drop the sizing border to change
//       the client size.  The app will adjust the new window size
//       so that the client's aspect ratio remains constant.  As a
//       result, the video remains properly proportioned, not
//       stretched or squashed, as the client size changes.
//
//
//    2) Select "Options" from the menu, then select "Mode", and
//       you see 2 modes.  One mode, the one you are already in,
//       is checked:  "Constant Aspect Ratio".  Choose the other
//       mode, "Adjust Source Area".
//
//    3) The sizing border disappears and a dialog box pops up.
//       The dialog box allows control of the video source
//       rectangle's size and shape.  You can ZOOM, which keeps
//       the source rectangle's aspect ratio constant while
//       changing the size.  Or you can STRETCH, which changes
//       the width or height of the source rectangle
//       independently: you get a stretched or squashed image.
//
//       Readouts show the current size of the video source
//       rectangle.
//
//    4) Other menu selections allow you to choose video window
//       sizes and shapes, change video input, control the
//       videodisc player, and display readouts of the sizes of
//       frame, client, destination, and source rectangles.
//
//
// C. Primitive Functions for Overlay and Videodisc Player Devices
//
//    VIDVIEW.C and VIDVIEW2.C include primitive functions to
//    do overlay and videodisc operations.
//
//    These are listed in subgroups in VIDVIEW.H.
//
//
//*****************************************************************************

#define INCL_WIN
#define INCL_WINSYS
#define INCL_WINFRAMEMGR
#define INCL_WINMENUS
#define INCL_WINDIALOGS
#define INCL_WINSTDDLGS
#define INCL_DOSMODULEMGR
#define INCL_GPI
#define INCL_MMCTL


#include <os2.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <os2me.h>

#include "MMCTL.H"
#include "LVDSYS.H"              // Custom videodisc player device info...
#include "vidview.h"             // Defines and function definitios for VIDVIEW

//-----------------------------------------------------------------------------
// Declares and initializations of global data...
//-----------------------------------------------------------------------------
HAB     hab        ;             // VIDVIEW anchor block...
HMQ     hmq        ;             // VIDIVEW message queue...
HWND    hwndFrame  ;             // Frame window handle
HWND    hwndClient ;             // Client window handle
HWND    hwndNotify = 0;          // Window to receive MCI_NOTIFY notifications...
HWND    hwndChild  ;             // Child of client
HWND    hwndMenu   ;             // Handle for Menu
HMODULE hMMCTLDll  ;             // MMCTL.DLL handle
BOOL    fShowWaitPointer=FALSE ; // When user has to wait...

// Information...
//    -- Player
//    -- Overlay
//    -- VIDVIEW
//    -- Initialize global pointers...
//
PLAYERINFO     apiPlayers [ 2 ] =
   {
   "Videodisc",   "VidviewVD1", 0, 0, 0, 0, 0, 0,  // We will use only this one player...
   "Videodisc02", "VidviewVD2", 0, 0, 0, 0, 0, 0,  // We could add this one for expansion...
   } ;
APPLICATIONINFO    aiMain ;
PPLAYERINFO        ppiCurrent = &apiPlayers  [ 0 ] ;
PAPPLICATIONINFO   paiMain    = &aiMain ;

// More initialization...
//    -- Readouts for client...
//    -- Device names for MCI control...
//    -- Boolean fDoFill indicating when to paint background of client...
//    -- Window sizes for PAL and NTSC shapes...
//    -- Window strings for PAL and NTSC window menu items...
//
DISPLAYVALUESINFO dviInfo [] =
   {
   "Frame: "       , "(%d, %d), cx=%d, cy=%d" ,"F:", "(%d,%d),%d,%d" ,NULL, TRUE,
   "V. window: "   , "(%d, %d), cx=%d, cy=%d" ,"V:", "(%d,%d),%d,%d" ,NULL, TRUE,
   "Destination: " , "(%d, %d), cx=%d, cy=%d" ,"D:", "(%d,%d),%d,%d" ,NULL, TRUE,
   "Source: "      , "(%d, %d), cx=%d, cy=%d" ,"S:", "(%d,%d),%d,%d" ,NULL, TRUE
   } ;

BYTE pbWidestNormalValue [] = { "(999, 999), cx=999, cy=999" };
BYTE pbWidestTerseValue  [] = { "(999,999),999,999" };

SZ    szOverlayDevName [] = "Overlay"   ;
SZ    szOverlayAlias   [] = "VidviewOV1";
SZ    szAmpMixAlias    [] = "VidviewAM1";
BOOL  fDoFill = TRUE ;

RECTL arclSrcRectsForNTSCInput [8] =
   {                                         // ---For NTSC:--------------
     8,   4, (688 + 8) - 1, (480 + 4) - 1,   // IDM_VDW_CHILD
     8,   4, (688 + 8) - 1, (480 + 4) - 1,   // IDM_VDW_CLIENT_FULL_EXTENT
     8,   4, (688 + 8) - 1, (480 + 4) - 1,   // IDM_VDW_CLIENT_04TH_EXTENT
     8,   4, (688 + 8) - 1, (480 + 4) - 1,   // IDM_VDW_CLIENT_16TH_EXTENT
     8,   4, (688 + 8) - 1, (480 + 4) - 1,   // IDM_VDW_CLIENT_64TH_EXTENT
     8,   4, (330 + 8) - 1, (473 + 4) - 1,   // IDM_VDW_CLIENT_ARBITRARY_1
     8,   4, (330 + 8) - 1, (473 + 4) - 1,   // IDM_VDW_CLIENT_ARBITRARY_2
     8,   4, (688 + 8) - 1, (344 + 4) - 1    // IDM_VDW_CLIENT_ARBITRARY_3
   } ;

RECTL arclDstRectsForNTSCInput [8] =
   {                                         // ---For NTSC:--------------
     0,   0, (172 + 0) - 1, (120 + 0) - 1,   // IDM_VDW_CHILD
     0,   0, (688 + 0) - 1, (480 + 0) - 1,   // IDM_VDW_CLIENT_FULL_EXTENT
     0,   0, (344 + 0) - 1, (240 + 0) - 1,   // IDM_VDW_CLIENT_04TH_EXTENT
     0,   0, (172 + 0) - 1, (120 + 0) - 1,   // IDM_VDW_CLIENT_16TH_EXTENT
     0,   0, ( 86 + 0) - 1, ( 60 + 0) - 1,   // IDM_VDW_CLIENT_64TH_EXTENT
     0,   0, (240 + 0) - 1, (344 + 0) - 1,   // IDM_VDW_CLIENT_ARBITRARY_1
     0,   0, (120 + 0) - 1, (172 + 0) - 1,   // IDM_VDW_CLIENT_ARBITRARY_2
     0,   0, (344 + 0) - 1, (172 + 0) - 1    // IDM_VDW_CLIENT_ARBITRARY_3
   } ;

RECTL arclSrcRectsForPALInput [8] =
   {                                         // ---For PAL:---------------
     8,  16, (688 + 8) - 1, (548 +16) - 1,   // IDM_VDW_CHILD
     8,  16, (688 + 8) - 1, (548 +16) - 1,   // IDM_VDW_CLIENT_FULL_EXTENT
     8,  16, (688 + 8) - 1, (548 +16) - 1,   // IDM_VDW_CLIENT_04TH_EXTENT
     8,  16, (688 + 8) - 1, (548 +16) - 1,   // IDM_VDW_CLIENT_16TH_EXTENT
     8,  16, (688 + 8) - 1, (548 +16) - 1,   // IDM_VDW_CLIENT_64TH_EXTENT
     8,  16, (449 + 8) - 1, (564 +16) - 1,   // IDM_VDW_CLIENT_ARBITRARY_1
     8,  16, (449 + 8) - 1, (564 +16) - 1,   // IDM_VDW_CLIENT_ARBITRARY_2
     8,  16, (688 + 8) - 1, (344 +16) - 1    // IDM_VDW_CLIENT_ARBITRARY_3
   } ;

RECTL arclDstRectsForPALInput [8] =
   {                                         // ---For PAL:---------------
     0,   0, (172 + 0) - 1, (137 + 0) - 1,   // IDM_VDW_CHILD
     0,   0, (688 + 0) - 1, (548 + 0) - 1,   // IDM_VDW_CLIENT_FULL_EXTENT
     0,   0, (344 + 0) - 1, (274 + 0) - 1,   // IDM_VDW_CLIENT_04TH_EXTENT
     0,   0, (172 + 0) - 1, (137 + 0) - 1,   // IDM_VDW_CLIENT_16TH_EXTENT
     0,   0, ( 86 + 0) - 1, ( 69 + 0) - 1,   // IDM_VDW_CLIENT_64TH_EXTENT
     0,   0, (274 + 0) - 1, (344 + 0) - 1,   // IDM_VDW_CLIENT_ARBITRARY_1
     0,   0, (137 + 0) - 1, (172 + 0) - 1,   // IDM_VDW_CLIENT_ARBITRARY_2
     0,   0, (344 + 0) - 1, (172 + 0) - 1    // IDM_VDW_CLIENT_ARBITRARY_3
   } ;


PSZ apszWindowStringsForNTSC [ 9 ] =
   {
   "C~hild of Client" ,
   "Client ~688 x 480",
   "Client ~344 x 240",
   "Client ~172 x 120",
   "Client ~86  x  60",
   "Client ~240 x 344",
   "Client 12~0 x 172",
   "Client 3~44 x 172",
   "~Window (NTSC)"
   } ;


PSZ apszWindowStringsForPAL [ 9 ] =
   {
   "C~hild of Client" ,
   "Client ~688 x 548",
   "Client ~344 x 274",
   "Client ~172 x 137",
   "Client ~86  x  69",
   "Client ~274 x 344",
   "Client 13~7 x 172",
   "Client 3~44 x 172",
   "~Window (PAL)"
   } ;




//-----------------------------------------------------------------------------
//  Name:         Main
//
//  Description:  Main procedure.
//
//  Concepts:     Registers window classes, creates application window,
//                opens video overlay device.
//
//  Parameters:   None.
//
//  Returns:      Returns 0 to the operating system.
//-----------------------------------------------------------------------------
int main ( VOID )
   {
   QMSG                 qmsg;
   APIRET               aRC;
   CHAR                 szFailBuffer[128];
   LONG                 i, VideoInput ;
   PPLAYERINFO          ppi ;
   PVIDEOWINDOWINFO     pviTemp, pviCurrent ;
   HPOINTER             hptrOriginal;
   HPOINTER             hptrWait;
   DWORD                dwReturn ;
   PSZ                  *ppsz ;
   LONG                 Index ;

   // Initialize and get an anchor block handle
   hab = WinInitialize ( 0 ) ;
   if ( !hab )
      {
      DosBeep ( BEEP_WARN_FREQ, BEEP_WARN_DUR ) ;
      exit ( RETURN_ERROR ) ;
      }

   aRC = DosLoadModule(szFailBuffer, sizeof(szFailBuffer), MMCONTROLS_DLLNAME, &hMMCTLDll);
   if(aRC != 0)
      return FALSE;

   // Create a message queue...
   hmq = WinCreateMsgQueue( hab, 0 );
   if ( !hmq)
      {
      DosBeep( BEEP_WARN_FREQ, BEEP_WARN_DUR );
      exit( RETURN_ERROR );
      }

   // Indicate that we are initializing...
   paiMain->fInitializing = TRUE ;

   // Get the sizing border sizes...
   paiMain->SizeBorderCx = WinQuerySysValue ( HWND_DESKTOP, SV_CXSIZEBORDER ) ;
   paiMain->SizeBorderCy = WinQuerySysValue ( HWND_DESKTOP, SV_CYSIZEBORDER ) ;

   // Get the screen size...
   paiMain->rclScreen.xLeft   = 0   ;
   paiMain->rclScreen.xRight  = WinQuerySysValue (HWND_DESKTOP,SV_CXSCREEN) -1;
   paiMain->rclScreen.yBottom = 0   ;
   paiMain->rclScreen.yTop    = WinQuerySysValue (HWND_DESKTOP,SV_CYSCREEN) -1;

   // Register the main window class...
   if ( WinRegisterClass ( hab,
                           "VidViewMainWindow",
                           MainWinProc,
                           CS_SIZEREDRAW | CS_MOVENOTIFY,
                           0 ) == FALSE )
      {
      ExitFunction ( "Could Not Register Class", RETURN_ERROR ) ;
      }

   // Register the child window class...
   if ( WinRegisterClass ( hab,
                           "VidViewChildWindow",
                           ChildWndProc,
                           CS_SIZEREDRAW | CS_MOVENOTIFY,
                           0 ) == FALSE )
      {
      ExitFunction ( "Could Not Register Class", RETURN_ERROR ) ;
      }

   // Initalize the strings required for opening the overlay device...
   strcpy ( paiMain->szOverlayDeviceName,  "Overlay"  ) ;
   strcpy ( paiMain->szOverlayDeviceAlias, "VidviewOV1" ) ;

   // Initialize other values in the application info structure...
   paiMain->SeekFrame  = 1 ;
   paiMain->fMinimized = FALSE ;
   paiMain->VideoInput = 1 ;

   // Display the Product Infomation dialog box...
   WinDlgBox( HWND_DESKTOP,
             HWND_DESKTOP,
             ( PFNWP )ProdInfoDlgProc,
             ( HMODULE ) 0,
             ID_DLG_PRODINFO,
             NULL );

   // Put up something for the user to look at...
   SetupMyMainWindow ( paiMain ) ;

   // Obtain handle of system pointer (cursor)
   hptrOriginal = WinQueryPointer( HWND_DESKTOP );

   // Obtain handle of system wait pointer (cursor)
   hptrWait = WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE );
   WinSetPointer( HWND_DESKTOP, hptrWait );

   // Initialize notify handle...
   hwndNotify = hwndClient ;

   // Open the overlay device...SHAREABLE
   if ( OpenDevice ( ( LPSTR )paiMain->szOverlayDeviceName,
                     ( LPSTR )paiMain->szOverlayDeviceAlias,
                     MCI_WAIT | MCI_OPEN_SHAREABLE,
                     &paiMain->wOverlayDeviceID ) )
      {
      DisplayMessage ("Open Overlay FAILED - DO NOT CONTINUE!");
      ExitFunction ("Device Opening Error", RETURN_ERROR ) ;
      return (TRUE);
      }

   // Initialize video source information and video display info for this app...
   for ( VideoInput = MAX_VIDEOWINDOWINFOS,
         pviTemp = &paiMain->aviVideoSources [ MAX_VIDEOWINDOWINFOS - 1 ] ;
         VideoInput > 0 ; VideoInput--, pviTemp-- )
      {
      InitializeVideoWindowInfo ( pviTemp, VideoInput ) ;
      }

   // Disable the "Window (NTSC)" or "Window (PAL)" menu item...
   // ...We are originally in maintain-client-aspect-ratio mode, and
   // in this mode we do not allow the "Window (NTSC)" (or
   // "Window (PAL)" for PAL) to be active...
   WinEnableMenuItem ( hwndMenu, IDM_VDW_MAIN, FALSE ) ;

   // Copy the video window info associated with input 1 into the
   // current video window block...and get a pointer to it...
   // ...Notice that we are actually copying the information from
   // one structure to the other, not just copying a pointer...
   paiMain->vwiInfo = paiMain->aviVideoSources [ paiMain->VideoInput - 1 ] ;
   pviCurrent = &paiMain->vwiInfo ;

   // In SetupMyMainWindow(), called above, we assumed that the
   // video input type for video input 1 was NTSC....If it was really
   // PAL (i.e. we opened the overlay device and queried the input type
   // and found it to be PAL) then update the frame and client to
   // reflect the default PAL sizes...

   if ( pviCurrent->InputType == VVK_IT_PAL )
      {
      // Set the frame rectangle position from the client position...
      SetFrameWindowPos ( hwndFrame, &paiMain->vwiInfo ) ;

      // Set the child window size...
      SetChildWindowPos ( hwndChild, &paiMain->vwiInfo ) ;

      // Set the text of the menu items to PAL values...
      for ( Index = 0, ppsz = pviCurrent->ppszWindowStrings ;
            Index < 9 ; Index++, ppsz++ )
         {
         WinSetMenuItemText ( hwndMenu, IDM_VDW_CHILD + Index, *ppsz ) ;
         }

      // Paint client window
      //WinUpdateWindow ( hwndClient ) ;
      }

   // Else the input type is NTSC...
   else
      {
      // Set the frame rectangle position from the client position...
      SetFrameWindowPos ( hwndFrame, &paiMain->vwiInfo ) ;
      }

   // Show the video...
   //
   //    1) Set the client background color to CLR_PINK so that video
   //       will show through...
   //
   //    2) Set source...
   //
   //    3) Set destination by directing video to the hwndClient...
   //
   pviCurrent->clrClientBG = CLR_PINK ;
   dwReturn = OvlPutSrcRect ( paiMain->wOverlayDeviceID, &pviCurrent->rclCurSrc, ( HWND ) 0 ) ;
   dwReturn = OvlSetVideoWindow ( paiMain->wOverlayDeviceID, hwndClient ) ;

   // Check the return code...and indicate whether we have succeeded...
   if ( LOUSHORT ( dwReturn ) == MCIERR_SUCCESS )
      {
      pviCurrent->fInitialized = TRUE ;
      }

   // Else we failed...
   else
      {
      DisplayMessage (
               "Cannot acquire the overlay device, because another "
               "application has acquired it exclusively.  If you "
               "want to see video in VIDVIEW, have the other application "
               "close the overlay device." ) ;
      }

   // Restore arrow pointer...
   WinSetPointer( HWND_DESKTOP, hptrOriginal );

   // Done initializing...
   paiMain->fInitializing = FALSE ;

   // Force a paint...
   WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
   WinUpdateWindow   ( hwndClient ) ;

   while ( WinGetMsg ( hab, &qmsg, ( HWND ) NULL, 0, 0 ) )
      {
      WinDispatchMsg( hab, &qmsg );
      }

   // Destroy the main window...
   WinDestroyWindow ( hwndFrame ) ;

   // Close any open devices...
   for ( i = 0, ppi = apiPlayers ; i < MAX_PLAYERS ; i++, ppi++ )
      {
      if ( ppi->wDeviceID != 0 )
         {
         CloseDevice ( ppi->wDeviceID ) ;
         }
      }
   if ( paiMain->wOverlayDeviceID != 0 )
      {
      CloseDevice ( paiMain->wOverlayDeviceID ) ;
      }

   // Destroy the message queue...
   // Terminate the application...
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );
   return( 0 );
   }



//-----------------------------------------------------------------------------
// Name:          SetupMyMainWindow
//
// Description:   Procedure for setting up the windows...
//
// Concepts:      Creates frame window and child window...
//
// Parameters:    pai - Pointer to main strcture for this app...
//
// Returns:       0
//
//-----------------------------------------------------------------------------
USHORT SetupMyMainWindow ( PAPPLICATIONINFO pai )
   {
   ULONG             flCreate = 0;
   PVIDEOWINDOWINFO  pvi = &pai->vwiInfo ;
   PRECTL            prclDstChild ;
   LONG              ChildWidth, ChildHeight ;

   // The initial mode is maintain client aspect ratio...
   // ...so we DO have a sizing border...
   flCreate    = FCF_STANDARD | FCF_NOBYTEALIGN ;

   // Create main frame window...invisibly...
   hwndFrame = WinCreateStdWindow ( HWND_DESKTOP,
                                    0,              // not WS_VISIBLE
                                    &flCreate,
                                    "VidViewMainWindow",
                                    "",
                                    0L,
                                    ( HMODULE )NULL,
                                    ID_MAINWND,
                                    &hwndClient );

   if ( !hwndFrame )
      {
      ExitFunction( "Could Not Create Main Window", RETURN_ERROR ) ;
      }

   // Set Frame window title text
   WinSetWindowText( hwndFrame, "Sample VIDVIEW" );

   // Initialize the video window information for the main window...
   // ...For now, assume NTSC input until we can open the overlay device...
   pvi->InputType = VVK_IT_NTSC ;
   pvi->prclSrcRects      = arclSrcRectsForNTSCInput ;
   pvi->prclDstRects      = arclDstRectsForNTSCInput ;
   pvi->ppszWindowStrings = apszWindowStringsForNTSC ;
   InitializeVideoDisplayInfo ( pvi, 1 ) ;

   // Set the frame rectangle position from the client position...
   SetFrameWindowPos ( hwndFrame, pvi ) ;

   // Create the child window
   prclDstChild = &pvi->prclDstRects [ 0 ] ;
   ChildWidth   = ( prclDstChild->xRight - prclDstChild->xLeft   ) + 1 ;
   ChildHeight  = ( prclDstChild->yTop   - prclDstChild->yBottom ) + 1 ;
   hwndChild = WinCreateWindow (
                     hwndClient,
                     "VidViewChildWindow",
                     NULL,
                     0,
                     128,
                     96,
                     ChildWidth,
                     ChildHeight,
                     hwndClient,
                     HWND_BOTTOM,
                     ID_STATIC,
                     NULL,
                     NULL ) ;

   // Paint client window
   WinUpdateWindow ( hwndClient ) ;
   return 0 ;
   }


//-----------------------------------------------------------------------------
// Name:          SetFrameWindowPos()
//
// Description:   Procedure for setting frame window size...
//
// Concepts:      Sets frame based on valued stored in pvi->rclClient...
//
// Parameters:    hwndFrame - frame window handle...
//                pvi - pointer to video window information...
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID SetFrameWindowPos ( HWND hwndFrame, PVIDEOWINDOWINFO pvi )
   {
   PRECTL prclFrame ;
   LONG   Height, Width ;

   prclFrame   = &pvi->rclFrame ;
   *prclFrame  = pvi->rclClient ;
   WinCalcFrameRect ( hwndFrame, prclFrame, FALSE ) ;

   // Get the height and width of the frame...
   Width  = ( prclFrame->xRight - prclFrame->xLeft   ) + 1 ;
   Height = ( prclFrame->yTop   - prclFrame->yBottom ) + 1 ;

   // Set the frame to the newly calculated size, at the current position...
   WinSetWindowPos ( hwndFrame, 0L ,
                     0,
                     0,
                     Width,
                     Height,
                     SWP_SHOW | SWP_SIZE ) ;

   // Query the frame rectangle coordinates....this is for debug...
   WinQueryWindowRect ( hwndFrame, prclFrame ) ;
   }


//-----------------------------------------------------------------------------
// Name:          SetChildWindowPos()
//
// Description:   Procedure for setting up the windows...
//
// Concepts:      Sets child window to current x, current y, new width,
//                new height
//
// Parameters:    hwndChild - handle of child window...
//                pvi       - pointer to video window information...
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID SetChildWindowPos ( HWND hwndChild, PVIDEOWINDOWINFO pvi )
   {
   PRECTL prclDstChild ;
   LONG   ChildWidth, ChildHeight ;

   prclDstChild = &pvi->prclDstRects [ 0 ] ;
   ChildWidth   = ( prclDstChild->xRight - prclDstChild->xLeft   ) + 1 ;
   ChildHeight  = ( prclDstChild->yTop   - prclDstChild->yBottom ) + 1 ;
   WinSetWindowPos ( hwndChild, 0L ,
                  0,
                  0,
                  ChildWidth,
                  ChildHeight,
                  SWP_SIZE ) ;
   }



//-----------------------------------------------------------------------------
// Name:          ExitFunction
//
// Description:   Displays a message and terminates VIDVIEW.EXE
//
// Concepts:      Use common function to display messages.
//
// Parameters:    Message text, error code w/ wh to exit.
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID ExitFunction ( PSZ pszMessage, WORD wErrorCode )
   {
   // Let the pointer symbol change back to the arrow icon...
   paiMain->fInitializing = FALSE ;

   // Display the message...
   DisplayMessage( pszMessage );

   // Exit with specified code...
   exit ( wErrorCode ) ;
   }

//-----------------------------------------------------------------------------
// Name:          CreateSourceControlDialog()
//
// Description:   Procedure for setting up the windows...
//
// Concepts:      Sets child window to current x, current y, new width,
//                new height
//
// Parameters:    pvi       - pointer to video window information...
//                fShow     - if TRUE show source control dialog...
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID CreateSourceControlDialog ( PVIDEOWINDOWINFO pvi, BOOL fShow )
   {
   RECTL rclFrame ;
   RECTL rclDlg ;
   LONG  DlgHeight, DlgWidth ;
   LONG  x, y ;
   HWND  hwndDlg ;
   LONG  SWPFlags ;

   // Create the dialog window...
   paiMain->hwndSourceDlg  = WinLoadDlg ( HWND_DESKTOP,
                                 HWND_DESKTOP,
                                 ( PFNWP )   SourceControlDialogProc,
                                 ( HMODULE ) NULL,
                                 ( ULONG )   ID_DLG_SOURCE_RECTANGLE,
                                 ( PVOID )   NULL ) ;

   //-------------------------------------------------------
   // The rest of this function adjusts the position of the
   // dialog window...
   //-------------------------------------------------------

   // Initialize...
   hwndDlg = paiMain->hwndSourceDlg ;

   // Get values for frame...
   rclFrame =  pvi->rclFrame ;
   WinMapWindowPoints ( hwndFrame, HWND_DESKTOP, ( PPOINTL ) &rclFrame, 2 ) ;
   rclFrame.xRight  -= 1 ;
   rclFrame.yTop    -= 1 ;

   // Get values for dlg...
   WinQueryWindowRect ( hwndDlg, &rclDlg ) ;
   DlgHeight = rclDlg.yTop   ;
   DlgWidth  = rclDlg.xRight ;
   WinMapWindowPoints ( hwndDlg, HWND_DESKTOP, ( PPOINTL ) &rclDlg, 2 ) ;

   // Initialize new (x,y) for dialog...
   x = rclDlg.xLeft   ;
   y = rclDlg.yBottom ;

   // If there is room to the right of the frame...
   // ...then position the dialog there...
   if (  ( paiMain->rclScreen.xRight - rclFrame.xRight )
       >= DlgWidth )
      {
      x = rclFrame.xRight + 1 ;
      y = ( rclFrame.yTop - DlgHeight ) + 1 ;
      }

   // Else if there is room to the bottom...then put it there...
   else if (  (rclFrame.yBottom - paiMain->rclScreen.yBottom)
            >= DlgHeight )
      {
      x = ( rclFrame.xRight  -  DlgWidth ) + 1 ;
      y = ( rclFrame.yBottom - DlgHeight ) ;
      }

   // Else if there is room to the left...then put it there...
   else if (  ( rclFrame.xLeft - paiMain->rclScreen.xLeft )
            >= DlgWidth )
      {
      x = rclFrame.xLeft - DlgWidth  ;
      y = ( rclFrame.yTop - DlgHeight ) + 1 ;
      }

   // Else if there is room to the top...then put it there...
   else if (  ( paiMain->rclScreen.yTop - rclFrame.yTop )
            >= DlgHeight )
      {
      x = ( rclFrame.xRight - DlgWidth ) + 1 ;
      y = rclFrame.yTop + 1 ;
      }

   // Else if there is almost enough room at the right...
   // ...then put it there...
   else if (  ( paiMain->rclScreen.xRight - rclFrame.xRight )
            >= ( ( DlgWidth * 3 ) / 4 ) )
      {
      x = ( paiMain->rclScreen.xRight - DlgWidth ) + 1 ;
      y = ( rclFrame.yTop - DlgHeight ) + 1 ;

      // Make sure we do not go above or below the screen...
      if (  ( ( y + DlgHeight ) - 1 )
          > paiMain->rclScreen.yTop )
         {
         y = paiMain->rclScreen.yTop - DlgHeight ;
         }
      else if ( y < paiMain->rclScreen.yBottom )
         {
         y = paiMain->rclScreen.yBottom ;
         }
      }

   // Else if there is almost enough room at the left...
   // ...then put it there...???

   // Arrange to show the dialog or not, as specified...
   if ( fShow )
      {
      SWPFlags = SWP_SHOW ;
      }
   else
      {
      SWPFlags = 0 ;
      }

   // Set the new position...
   WinSetWindowPos ( hwndDlg, 0L, x, y, 0, 0, SWPFlags | SWP_MOVE ) ;
   }


//-----------------------------------------------------------------------------
// Name:          DisplayValuesInClientArea()
//
// Description:   Displays readouts for frame, client, destination, and source
//
// Concepts:      Solicits current values from overlay device.
//
//                Note on destination:
//
// Parameters:    hps       - hps to draw in
//                pvi       - pointer to video window structure
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID DisplayValuesInClientArea ( HPS hps, PVIDEOWINDOWINFO pvi )
{
   RECTL  rclTempFrame, rclTempSrc, rclTempDst, rclTempVideo ;
   RECTL  rclWidth, rclLabel, rclValue ;
   PRECTL prcl ;
   BYTE   abTemp [ 60 ] ;
   LONG   i, TotalLineWidth ;
   PDISPLAYVALUESINFO pdi ;
   HWND hwndVideo ;
   PBYTE  pbLabel, pbValue ;
   BOOL   fTerseFormat ;

   // If we have not previously computed the text rectangle widths
   // then do so now...
   if ( paiMain->NormalLabelWidth == 0)
      {
      // Get max width of normal label....
      rclWidth.xLeft   = 0 ;
      rclWidth.yBottom = 0 ;
      rclWidth.xRight  = 200 ;
      rclWidth.yTop    = 200 ;
      WinDrawText ( hps, -1, dviInfo[1].pbNormalLabel, &rclWidth,
                          pvi->clrClientFG,
                          pvi->clrClientBG,
                          DT_VCENTER | DT_LEFT | DT_QUERYEXTENT ) ;
      paiMain->NormalLabelWidth = rclWidth.xRight - rclWidth.xLeft ;

      // Get max width of normal value string...
      rclWidth.xLeft   = 0 ;
      rclWidth.yBottom = 0 ;
      rclWidth.xRight  = 200 ;
      rclWidth.yTop    = 200 ;
      WinDrawText ( hps, -1, pbWidestNormalValue, &rclWidth,
                          pvi->clrClientFG,
                          pvi->clrClientBG,
                          DT_VCENTER | DT_LEFT | DT_QUERYEXTENT ) ;
      paiMain->NormalValueWidth = rclWidth.xRight - rclWidth.xLeft ;

      // Get max width of terse label....
      rclWidth.xLeft   = 0 ;
      rclWidth.yBottom = 0 ;
      rclWidth.xRight  = 200 ;
      rclWidth.yTop    = 200 ;
      WinDrawText ( hps, -1, dviInfo[1].pbTerseLabel, &rclWidth,
                          pvi->clrClientFG,
                          pvi->clrClientBG,
                          DT_VCENTER | DT_LEFT | DT_QUERYEXTENT ) ;
      paiMain->TerseLabelWidth = rclWidth.xRight - rclWidth.xLeft ;

      // Get max width of terse value string...
      rclWidth.xLeft   = 0 ;
      rclWidth.yBottom = 0 ;
      rclWidth.xRight  = 200 ;
      rclWidth.yTop    = 200 ;
      WinDrawText ( hps, -1, pbWidestTerseValue, &rclWidth,
                          pvi->clrClientFG,
                          pvi->clrClientBG,
                          DT_VCENTER | DT_LEFT | DT_QUERYEXTENT ) ;
      paiMain->TerseValueWidth = rclWidth.xRight - rclWidth.xLeft ;

      }

   // Init pointers to rectangles....
   dviInfo[0].prclValues = &rclTempFrame ;
   dviInfo[1].prclValues = &rclTempVideo ;
   dviInfo[2].prclValues = &rclTempDst   ;
   dviInfo[3].prclValues = &rclTempSrc   ;

   // Get frame window coordinates...and map to desktop...
   rclTempFrame = pvi->rclFrame ;
   WinMapWindowPoints ( hwndFrame, HWND_DESKTOP, ( PPOINTL ) &rclTempFrame, 2 );
   rclTempFrame.xRight-- ;
   rclTempFrame.yTop--   ;

   // Get video window coordinates...and map to frame coordinates
   if ( pvi->Style == IDM_VDW_CHILD )
      {
      WinQueryWindowRect ( hwndChild, &rclTempVideo ) ;
      rclTempVideo.xRight-- ;
      rclTempVideo.yTop--   ;
      hwndVideo = hwndChild ;
      }
   else
      {
      rclTempVideo = pvi->rclClient ;
      //rclTempVideo.xRight-- ;     // For client we have already decremented this value
      //rclTempVideo.yTop--   ;     // For client we have already decremented this value
      hwndVideo    = hwndClient ;
      }
   WinMapWindowPoints ( hwndVideo, hwndFrame, ( PPOINTL ) &rclTempVideo, 2 ) ;

   // Fetch the destination and source as reported by overlay device...
   OvlGetDstRect ( paiMain->wOverlayDeviceID, ( HWND ) 0, &rclTempDst ) ;
   OvlGetSrcRect ( paiMain->wOverlayDeviceID, ( HWND ) 0, &rclTempSrc ) ;

   // Get ready to paint....
   GpiSetBackMix ( hps, BM_OVERPAINT ) ;

   // Determine the total width of the text...
   TotalLineWidth = paiMain->NormalLabelWidth + paiMain->NormalValueWidth ;

   // If displaying...
   if ( pvi->Readout != VVK_RO_NONE )
      {

      // Initialize label rectangle for normal format...
      rclLabel.xLeft   = VVK_DR_XLEFT                             ;
      rclLabel.xRight  = VVK_DR_XLEFT + paiMain->NormalLabelWidth ;
      rclLabel.yBottom = VVK_DR_YBOTTOM                           ;
      rclLabel.yTop    = VVK_DR_YTOP                              ;

      // Initialize value rectangle...
      rclValue.xLeft   = VVK_DR_XLEFT + paiMain->NormalLabelWidth + 1;
      rclValue.xRight  = VVK_DR_XLEFT + TotalLineWidth ;
      rclValue.yBottom = VVK_DR_YBOTTOM ;
      rclValue.yTop    = VVK_DR_YTOP    ;

      // Check for terse format...
      if ( ( ( pvi->rclClient.xRight - pvi->rclClient.xLeft) + 1 )
          >= TotalLineWidth )
         {
         fTerseFormat    = FALSE ;
         }
      else
         {
         fTerseFormat    = TRUE ;
         rclLabel.xRight = VVK_DR_XLEFT + paiMain->TerseLabelWidth ;
         rclValue.xLeft  = VVK_DR_XLEFT + paiMain->TerseLabelWidth + 1;
         rclValue.xRight = VVK_DR_XLEFT + paiMain->TerseLabelWidth
                                        + paiMain->TerseValueWidth ;
         }

      // Display 4 lines of text...
      for ( i = 0, pdi = dviInfo; i < 4 ; i++, pdi++ )
         {
         // Adjust...
         rclLabel.yBottom -= 15 ;
         rclLabel.yTop    -= 15 ;
         rclValue.yBottom -= 15 ;
         rclValue.yTop    -= 15 ;
         prcl = pdi->prclValues ;

         // Get pointers to the strings...
         if ( fTerseFormat )
            {
            pbLabel = pdi->pbTerseLabel ;
            pbValue = pdi->pbTerseValue ;
            }
         else
            {
            pbLabel = pdi->pbNormalLabel ;
            pbValue = pdi->pbNormalValue ;
            }

         // Display the label part of the line...
         WinDrawText ( hps, -1, pbLabel, &rclLabel,
                       pvi->clrClientFG,
                       pvi->clrClientBG,
                       DT_VCENTER | DT_LEFT | DT_ERASERECT ) ;

         // Display the value part of the line...
         sprintf ( abTemp, pbValue,
                   prcl->xLeft,
                   prcl->yBottom,
                   ( prcl->xRight - prcl->xLeft ) + 1,
                   ( prcl->yTop - prcl->yBottom ) + 1 ) ;
         WinDrawText ( hps, -1, abTemp, &rclValue,
                       pvi->clrClientFG,
                       pvi->clrClientBG,
                       DT_VCENTER | DT_LEFT | DT_ERASERECT ) ;
         }
      }

   // Else erasing...
   else
      {
      // Initialize drawing rectangle...
      rclLabel.xLeft   = VVK_DR_XLEFT   ;
      rclLabel.xRight  = VVK_DR_XLEFT + TotalLineWidth ;
      rclLabel.yBottom = 10 ;
      rclLabel.yTop    = 70 ;
      WinFillRect ( hps, &rclLabel, pvi->clrClientBG ) ;
      }
}


//-----------------------------------------------------------------------------
// Name:          MapReadoutToClr()
//
// Description:   Map a color to a menu item id...
//
// Concepts:      Map color value to menu value
//
// Parameters:    Readout - value indicating readout color
//                clr     - PM Color
//
// Returns:       COLOR
//
//-----------------------------------------------------------------------------
VOID MapReadoutToClr ( LONG Readout, PCOLOR pclr )
   {
   COLOR clr ;

   // Get the color associated w/ the readout value...
   switch ( Readout )
      {
      case VVK_RO_WHITE :
         clr = CLR_WHITE ;
         break ;

      case VVK_RO_YELLOW :
         clr = CLR_YELLOW ;
         break ;

      case VVK_RO_CYAN :
         clr = CLR_CYAN ;
         break ;

      case VVK_RO_BLACK :
         clr = CLR_BLACK ;
         break ;

      default :
         clr = CLR_PINK ;
         break ;
      }

   // Set return value...
   if ( clr != CLR_PINK )
      {
      *pclr = clr ;
      }
   }


//-----------------------------------------------------------------------------
// Name:          BuildProRect()
//
// Description:   Given a height or width, build a rectangle proportional to
//                the specified aspect ratio.
//
// Concepts:      For video to appear properly proportioned, not stretched
//                or squashed, the aspect ratio (width-to-height ratio)
//                of the source rectangle must be the same, or at least
//                approximately the same, as the aspect ratio of the
//                destination ratio.
//
//                Here we are given an aspect ratio and one dimension,
//                either length or height.  From this we calculate
//                the other dimension.
//
// Parameters:    prclOld - input
//                prclNew - the new non-proportional rectangle
//                prclPro - output, prclNew made proportional
//                AspectX, AspectY - Aspect width and height; JUST USE THE
//                                   ACTUAL DIMENSIONS OF YOUR SOURCE RECTANGLE
//                NewWidth, NewHeight - One of these (the non-0 one) specifies
//                                      whether we are to start w the height
//                                      and determine the width, or vice versa
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID BuildProRect ( PRECTL prclOld, PRECTL prclNew, PRECTL prclPro,
                        LONG AspectX, LONG AspectY,
                        LONG NewWidth, LONG NewHeight )
{
LONG ProWidth, ProHeight ;

   // If we are given the width and have to find the height...
   if ( NewWidth != 0 )
      {
      // Get the new proportional width...which is the same as the new width...
      ProWidth = NewWidth ;

      // Calculate the new proportional height using
      // the proportional width and the constant aspect ratio...
      ProHeight = ( ( ProWidth * AspectY ) + ( AspectX / 2 ) ) / AspectX ;
      }

   // Else we are given the height and have to find the width...
   else
      {
      // Get the new proportional height...which is the same as the new height..
      ProHeight = NewHeight ;

      // Calculate the new proportional width using
      // the proportional height and the constant aspect ratio...
      ProWidth = ( ( ProHeight * AspectX ) + ( AspectY / 2 ) ) / AspectY ;
      }

   // Now build the width of the new rectangle...
   // If the left edges are the same then build the new
   // rectangle from the left...
   if ( prclOld->xLeft == prclNew->xLeft)
      {
      prclPro->xLeft  = prclNew->xLeft ;
      prclPro->xRight = prclPro->xLeft + ( ProWidth - 1 ) ;
      }

   // Else the right edges are the same...so build the new
   // rectangle from the right...
   else
      {
      prclPro->xRight = prclNew->xRight ;
      prclPro->xLeft  = prclPro->xRight - ( ProWidth - 1 ) ;
      }

   // Now build the height of the new rectangle...
   // If the bottom edges are the same then build the new
   // rectangle from the bottom...
   if ( prclOld->yBottom == prclNew->yBottom)
      {
      prclPro->yBottom = prclNew->yBottom ;
      prclPro->yTop    = prclPro->yBottom + ( ProHeight - 1 ) ;
      }

   // Else the top edge is the same...so build the new
   // rectangle from the top...
   else
      {
      prclPro->yTop    = prclNew->yTop ;
      prclPro->yBottom = prclPro->yTop - ( ProHeight - 1 ) ;
      }
}


//-----------------------------------------------------------------------------
// Name:          MapRectToProRect()
//
// Description:   Map rectangle to proportional rectangle.  Choose the new
//                rectangle based on the direction of greatest change.
//
// Concepts:      Used when in maintain-client-aspect-ratio mode.
//                The user has sized the rectangle in both width and
//                height (altho the change may be 0 in either dimension).
//                We select the dimension of greater change and create
//                a new proportional rectangle based on it.
//
// Parameters:    prclOld - Old size
//                prclNew - New non-prop size
//                prclPro - Output, the new prop size
//                AspectX, AspectY - the aspect ratio; USE ACTUAL DIMENSIONS
//                                   OF SOURCE RECTANGLE
//
// Returns:       TRUE success
//                FALSE fail
//
//-----------------------------------------------------------------------------
BOOL MapRectToProRect ( PRECTL prclOld, PRECTL prclNew, PRECTL prclPro,
                        LONG AspectX, LONG AspectY,
                        LONG MinWidth, LONG MinHeight,
                        LONG MaxWidth, LONG MaxHeight )
{
   LONG  OldWidth, NewWidth, OldHeight, NewHeight, ScreenWidth, ScreenHeight ;
   LONG  ChangeInWidth, ChangeInHeight ;

   // Get the dimensions...either or both members of a coordinate pair
   // may be posative,negative, or 0...
   OldWidth  = ( prclOld->xRight - prclOld->xLeft    ) + 1 ;
   OldHeight = ( prclOld->yTop   - prclOld->yBottom  ) + 1 ;
   NewWidth  = ( prclNew->xRight - prclNew->xLeft    ) + 1 ;
   NewHeight = ( prclNew->yTop   - prclNew->yBottom  ) + 1 ;
   ChangeInWidth  = NewWidth  - OldWidth  ;
   if ( ChangeInWidth < 0 )
      {
      ChangeInWidth = -( ChangeInWidth ) ;
      }
   ChangeInHeight = NewHeight - OldHeight ;
   if ( ChangeInHeight < 0 )
      {
      ChangeInHeight = -( ChangeInHeight ) ;
      }

   // If the old width is 0 then return now....
   // NOTE: The cx and cy reported by WM_SIZE during set up is 0 and 0,
   //       because the window is hidden...
   if ( OldWidth == 0 )
      {
      return FALSE ;
      }

   // If the width changed more than the height...
   // NOTE:  For the purpose of this comparison ONLY, we will factor in
   //        the SCREEN dimensions so that the user will
   //        be more comfortable with the changes in client size...
   //
   ScreenWidth  = (paiMain->rclScreen.xRight - paiMain->rclScreen.xLeft  ) +1 ;
   ScreenHeight = (paiMain->rclScreen.yTop   - paiMain->rclScreen.yBottom) +1 ;
   ChangeInHeight = (   ( ChangeInHeight * ScreenHeight )
                      + ( ScreenWidth / 2 ) ) / ScreenWidth ;
   if ( ChangeInWidth >= ChangeInHeight )
      {
      // Enforce minimum and maximum width...
      if ( NewWidth < MinWidth )
         {
         NewWidth = MinWidth ;
         }
      else if ( NewWidth > MaxWidth )
         {
         NewWidth = MaxWidth ;
         }


      // Build new proportional rectangle...
      BuildProRect ( prclOld, prclNew, prclPro, AspectX, AspectY,
                     NewWidth, 0 ) ;
      }
   // Else we are to use the new height...
   else
      {
      // Enforce minimum and maximum height...
      if ( NewHeight < MinHeight )
         {
         NewHeight = MinHeight ;
         }
      else if ( NewHeight > MaxHeight )
         {
         NewHeight = MaxHeight ;
         }
      BuildProRect ( prclOld, prclNew, prclPro, AspectX, AspectY,
                     0, NewHeight ) ;
      }
return TRUE ;
}


//-----------------------------------------------------------------------------
// Name:          ChangeActionMode()
//
// Descciption:   Switch between Maintain-client-aspect-ratio mode and
//                Adjust-source-area mode.  We have to add or discard the
//                frame sizing border, re-size the window, and update the
//                menu check marks.
//
// Concepts:      When switching modes we use a default source size
//                and a default destination size...
//
// Parameters:    Mode - which is the new mode
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID ChangeActionMode ( PVIDEOWINDOWINFO pvi, LONG Mode )
   {
   LONG        Width, Height, StyleMask, Active, Inactive, EnableItems ;
   PRECTL      prclFrame, prclClient ;
   RECTL       rclOldFrame ;
   LONG        xLeft, yBottom ;

   // Set the new mode...
   pvi->ActionMode = Mode ;

   // Set temporary variables based on the new mode...
   if ( Mode == VVK_AM_ADJUST_SOURCE_AREA )
      {
      StyleMask          = 0 ;
      Active             = IDM_OVLY_ADJUST_SOURCE_AREA ;
      Inactive           = IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO ;
      EnableItems        = TRUE ;
      }
   else
      {
      StyleMask          = FS_SIZEBORDER ;
      Active             = IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO ;
      Inactive           = IDM_OVLY_ADJUST_SOURCE_AREA ;
      EnableItems        = FALSE ;
      }

   // Get the old frame position in world coordinates...
   WinQueryWindowRect ( hwndFrame, &rclOldFrame ) ;
   WinMapWindowPoints ( hwndFrame, HWND_DESKTOP, ( PPOINTL ) &rclOldFrame, 2 ) ;

   // Add or remove sizing border...
   WinSetWindowBits ( hwndFrame, QWL_STYLE, StyleMask, FS_SIZEBORDER ) ;

   // Set pointers...
   prclFrame   = &pvi->rclFrame  ;
   prclClient  = &pvi->rclClient ;

   // Set client to default size...
   *prclClient = pvi->prclDstRects[IDM_VDW_CLIENT_04TH_EXTENT-IDM_VDW_CHILD] ;

   // Set minimum source rectangle to same size as destination...
   pvi->rclMinSrc  = *prclClient ;

   // Calculate the equivalent frame rectangle...and get its width and height...
   *prclFrame = *prclClient ;
   WinCalcFrameRect ( hwndFrame, prclFrame, FALSE ) ;
   Width  = ( prclFrame->xRight - prclFrame->xLeft   ) + 1 ;
   Height = ( prclFrame->yTop   - prclFrame->yBottom ) + 1 ;

   // Get the old frame coordinates...
   xLeft   = rclOldFrame.xLeft   ;
   yBottom = rclOldFrame.yBottom ;

   // Adjust for size of sizing border...
   if ( Mode == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
      {
      xLeft    -= paiMain->SizeBorderCx ;
      yBottom  -= paiMain->SizeBorderCy ;
      }
   else
      {
      xLeft    += paiMain->SizeBorderCx ;
      yBottom  += paiMain->SizeBorderCy ;
      }

   // Set default source rectangle...
   SetDefaultSourceRect ( pvi ) ;

   // Set the frame to the new calculated size, at the old position...
   WinSetWindowPos ( hwndFrame, 0L ,
                     xLeft, yBottom, Width, Height,
                     SWP_SHOW | SWP_SIZE | SWP_MOVE ) ;

   // Update the frame...
   WinSendMsg ( hwndFrame, WM_UPDATEFRAME, ( MPARAM) FCF_SIZEBORDER,
                ( MPARAM ) 0 ) ;

   // Update menu check marks...
   WinCheckMenuItem ( hwndMenu, Active, TRUE ) ;
   WinCheckMenuItem ( hwndMenu, Inactive, FALSE ) ;

   // Enable or disable video window menuitem...
   WinEnableMenuItem ( hwndMenu, IDM_VDW_MAIN, EnableItems ) ;

   // Uncheck the old video window style...
   WinCheckMenuItem ( hwndMenu, pvi->Style, FALSE ) ;

   // If the new mode is maintain-client-aspect-ratio...
   if ( Mode == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
      {

      // If switching FROM child window...
      if ( pvi->Style == IDM_VDW_CHILD )
         {

         // Set the new colors...
         //    -- Foreground color does not change...
         //    -- Background color is CLR_PINK...
         pvi->clrClientBG = CLR_PINK ;

         // Force a fill of the client area...
         fDoFill = TRUE ;

         // Hide the child window...and use the client as the video window...
         WinSetWindowPos ( hwndChild, 0L , 0, 0, 0, 0, SWP_HIDE ) ;
         OvlSetVideoWindow ( paiMain->wOverlayDeviceID, hwndClient ) ;

         // Re-enable the "White" selection on the readout colors menu...
         WinEnableMenuItem ( hwndMenu, IDM_RDTCLR_WHITE, TRUE ) ;
         }
      }

   // Set the new video window style...
   pvi->Style = IDM_VDW_CLIENT_04TH_EXTENT ;

   // Check the new video window style...
   if ( Mode == VVK_AM_ADJUST_SOURCE_AREA )
      {
      WinCheckMenuItem ( hwndMenu, IDM_VDW_CLIENT_04TH_EXTENT, TRUE ) ;
      }

   // Get the frame rectangle coordinates....
   WinQueryWindowRect ( hwndFrame, prclFrame ) ;

   // Paint the client window
   WinUpdateWindow ( hwndClient ) ;
   }


//-----------------------------------------------------------------------------
// Name:          ChildWndProc()
//
// Description:   Processes WM_PAINT for child window.  Necessary for displaying
//                video in our child window -- video shows through only CLR_PINK.
//
// Concepts:      It is comparatively very easy to manage painting the video
//                window when it contains ONLY video -- just flood with
//                CLR_PINK.  If the video window is partially covered by
//                children (for example, in this program, when the video
//                window is the client, and the numerical readouts are
//                display) then we can encounter some annoying problems
//                with flickering text.
//
//                Remember, w/ a video window, you get MORE FREQUENT PAINTS,
//                and MORE EXTENSIVE PAINTS (i.e. often ALL of the window
//                is invalidated), as a result of the overlay device's
//                continuing responsibility to keep the video window
//                covered with current video.  Therefore it is better if
//                you do NOT display any other controls over the video window.
//
// Parameters:    Std window proc parms
//
// Returns:       Std window proc returns
//
//-----------------------------------------------------------------------------
MRESULT EXPENTRY ChildWndProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   HPS   hps ;
   RECTL rcl ;

   switch ( msg )
      {
      case WM_PAINT:

         hps = WinBeginPaint ( hwnd, (HPS)NULL, &rcl ) ;
         WinFillRect ( hps, &rcl, CLR_PINK ) ;
         WinEndPaint ( hps ) ;
         break;

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }


//-----------------------------------------------------------------------------
// Name:          MainWndProc
//
// Description:   Window procedure for the main clent window.
//
// Concepts :     Processes the messages sent to the main client
//                window.  This routine processes the basic
//                messages all client windows should process and
//                passes all others onto UserWndProc where the
//                developer can process any others.
//
//                Where this function is complex is NOT in the management of
//                overlay, which is relatively simple, but in maintaining size
//                calculations for for the adjust-source-area popup.
//
// Parameters:    Std window proc parms
//
// Returns:       Std window proc returns
//
//-----------------------------------------------------------------------------
MRESULT EXPENTRY MainWinProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   BOOL              fFailed ;
   USHORT            usCommand  ;
   DWORD             dwMciFlags ;
   DWORD             dwAudio ;
   PSZ               pszText ;
   PRECTL            prcl, prclMin, prclMax ;
   RECTL             rclNew, rclResult, rclNewFrame ;
   RECTL             rclOldClient, rclOldClientOnDesktop ;
   BOOL              fSuccess, fCheck ;
   LONG              OldVideoInput, NewVideoInput, OldReadout ;
   LONG              MinWidth, MinHeight, MaxWidth, MaxHeight ;
   DWORD             dwReturn ;
   PSWP              pswp ;
   PVIDEOWINDOWINFO  pvi ;

   switch ( msg )
      {

      case WM_CREATE:

         // Get the menu handle...
         hwndMenu = WinWindowFromID (
                         WinQueryWindow ( hwnd, QW_PARENT ),
                         FID_MENU ) ;

         // Put a check beside the "Constant Aspect Ratio"
         // on the menu...
         WinCheckMenuItem ( hwndMenu, IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO,
                            TRUE ) ;

         // Put a check beside the video input 1...
         WinCheckMenuItem ( hwndMenu, IDM_VID_INPUT_1, TRUE ) ;

         // Put a check beside "Off" on the "Readouts" submenu...
         WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_NONE, TRUE ) ;

         // Disable video window menuitems...
         WinEnableMenuItem ( hwndMenu, IDM_VDW_MAIN, FALSE ) ;

         // Disable the player menu selections...
         WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_FORWARD, FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_REVERSE, FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_STOP        , FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_SEEK        , FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_INFO_DISPLAY, FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_AUDIO       , FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_STATUS      , FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_SPIN        , FALSE ) ;
         WinEnableMenuItem ( hwndMenu, IDM_VDP_CLOSE_DEVICE, FALSE ) ;
         break;


      case WM_ACTIVATE:

         // If we are activating...
         if   ( SHORT1FROMMP ( mp1 ) )
            {
            // Check the videodisc player device...
            if ( ppiCurrent != NULL )
               {
               // If the device is open and we do not own it...then acquire it...
               if ( (ppiCurrent->wDeviceID != 0 ) && ( !ppiCurrent->fOwnDevice ) )
                  {
                  AcquireDevice ( ppiCurrent->wDeviceID, 0 ) ;
                  }
               }
            }
         break;


      case WM_HELP:
         // Display help window when F1 is pressed...
         WinDlgBox(HWND_DESKTOP, hwnd,
               DevHelpInfoDlgProc,
               (HMODULE) 0, ID_HELP_INFO_DLG,
               0);
         break;


      // ***************************
      // *** Menu item selections...
      // ***************************
      case WM_COMMAND:
         // Initialize pointer to video window info...
         pvi = &paiMain->vwiInfo ;

         // Extract the command value
         usCommand = SHORT1FROMMP (mp1);
         switch ( usCommand )
            {

            //***************************************
            //*** MODE Submenu...
            //***************************************

            case IDM_OVLY_ADJUST_SOURCE_AREA:

               // Show the wait pointer...
               fShowWaitPointer = TRUE ;
               WinSetPointer( HWND_DESKTOP,
                              WinQuerySysPointer(HWND_DESKTOP,SPTR_WAIT,FALSE));

               // If we are NOT already in adjust-source-size mode...
               // ...then change to that mode...
               if ( pvi->ActionMode != VVK_AM_ADJUST_SOURCE_AREA )
                  {
                  ChangeActionMode ( pvi, VVK_AM_ADJUST_SOURCE_AREA ) ;
                  }

               // If the source control dialog is not already active...
               // ...then make it active...
               if ( paiMain->hwndSourceDlg == 0 )
                  {
                  // Create the source control dialog...
                  CreateSourceControlDialog ( pvi, TRUE ) ;
                  }

               // Clear the wait pointer...
               fShowWaitPointer = FALSE ;
               break;


            case IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO:

               // If we are NOT already in maintain-client-aspect-ration mode...
               if ( pvi->ActionMode != VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
                  {
                  // Show the wait pointer...
                  fShowWaitPointer = TRUE ;
                  WinSetPointer( HWND_DESKTOP,
                                 WinQuerySysPointer(HWND_DESKTOP,SPTR_WAIT,FALSE));

                  // Close the source control dialog if it is active...
                  if ( paiMain->hwndSourceDlg )
                     {
                     WinDestroyWindow ( paiMain->hwndSourceDlg ) ;
                     }

                  // Change to the new action mode...
                  ChangeActionMode ( &paiMain->vwiInfo, VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO ) ;

                  // Clear the wait pointer...
                  fShowWaitPointer = FALSE ;
                  }
               break;


            //*************************************
            //*** PLAYER SUBMENU OPTIONS
            //*************************************

            case IDM_VDP_OPEN_DEVICE :

               // If already open then display a message...
               if ( ppiCurrent->wDeviceID != 0 )
                  {
                  DisplayMessage ("Player already opened");
                  }


               // Else try the open...
               else
                  {
                  // Show the wait pointer...
                  fShowWaitPointer = TRUE ;
                  WinSetPointer( HWND_DESKTOP,
                              WinQuerySysPointer(HWND_DESKTOP,SPTR_WAIT,FALSE));

                  // Try the open...
                  fFailed = OpenDevice ( ( LPSTR )ppiCurrent->szDeviceName,
                                 ( LPSTR )ppiCurrent->szDeviceAlias,
                                 MCI_WAIT | MCI_OPEN_SHAREABLE,
                                 &ppiCurrent->wDeviceID ) ;

                  // Clear the wait pointer...
                  fShowWaitPointer = FALSE ;

                  // If the open failed...
                  if ( fFailed )
                     {
                     // Tell the user...
                     DisplayMessage ("Open Player FAILED");
                     }
                  else
                     {
                     // Remember some values for this player...These are the default
                     // values upon opening...
                     ppiCurrent->fAudioAllOn    = TRUE  ;
                     ppiCurrent->fInfoDisplayOn = FALSE ;
                     ppiCurrent->fDoorClosed    = TRUE  ;
                     ppiCurrent->fSpunUp        = TRUE  ;

                     // Disable the "open device" menu selection...
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_OPEN_DEVICE, FALSE ) ;

                     // Enable the player menu selections...
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_FORWARD, TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_REVERSE, TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_STOP        , TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_SEEK        , TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_INFO_DISPLAY, TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_AUDIO       , TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_STATUS      , TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_SPIN        , TRUE ) ;
                     WinEnableMenuItem ( hwndMenu, IDM_VDP_CLOSE_DEVICE, TRUE ) ;

                     // Set check on for audio....is on upon open...
                     WinCheckMenuItem ( hwndMenu, IDM_VDP_AUDIO, TRUE  ) ;
                     }
                  }
               break;

            case IDM_VDP_CLOSE_DEVICE :

               // If not open then display a message...
               if ( ppiCurrent->wDeviceID == 0 )
                  {
                  DisplayMessage ("Cannot close device, not open");
                  }

               // Else close...
               else
                  {
                  // Do the close...
                  CloseDevice ( ppiCurrent->wDeviceID ) ;

                  // Reset the handle...
                  ppiCurrent->wDeviceID = 0 ;

                  // Enable the "open device" menu selection...
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_OPEN_DEVICE, TRUE ) ;

                  // Disable the other player menu selections...
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_FORWARD, FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_PLAY_REVERSE, FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_STOP        , FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_SEEK        , FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_INFO_DISPLAY, FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_AUDIO       , FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_STATUS      , FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_SPIN        , FALSE ) ;
                  WinEnableMenuItem ( hwndMenu, IDM_VDP_CLOSE_DEVICE, FALSE ) ;

                  // Uncheck audio and info display
                  WinCheckMenuItem ( hwndMenu, IDM_VDP_AUDIO       , FALSE ) ;
                  WinCheckMenuItem ( hwndMenu, IDM_VDP_INFO_DISPLAY, FALSE ) ;
                  }
               break;


            case IDM_VDP_PLAY_FORWARD :
               if ( VdpReady ( ppiCurrent->wDeviceID ) )
                  {
                  VdpPlay ( ppiCurrent->wDeviceID, TRUE, 0, 0 ) ;
                  }
               break;

            case IDM_VDP_PLAY_REVERSE :
               if ( VdpReady ( ppiCurrent->wDeviceID ) )
                  {
                  VdpPlay ( ppiCurrent->wDeviceID, FALSE, 0, 0 ) ;
                  }
               break;

            case IDM_VDP_PLAY_FROM_TO   :
               break;

            case IDM_VDP_STOP   :
               if ( VdpReady ( ppiCurrent->wDeviceID ) )
                  {
                  VdpStop ( ppiCurrent->wDeviceID ) ;
                  }
               break;

            case IDM_VDP_SEEK          :

               if ( WinDlgBox  ( HWND_DESKTOP,
                                 hwndFrame,
                                 ( PFNWP )   SeekDlgProc,
                                 ( HMODULE ) NULL,
                                 ( ULONG )   ID_DLG_SEEK,
                                 ( PVOID )   NULL ) )
                  {
                  if ( VdpReady ( ppiCurrent->wDeviceID ) )
                     {
                     VdpSeek ( ppiCurrent->wDeviceID, paiMain->SeekFrame,0,0,0);
                     }
                  }
               break ;

            case IDM_VDP_INFO_DISPLAY :
               // If info display is ON then set it OFF...
               // ...and vice versa...
               if ( ppiCurrent->fInfoDisplayOn )
                  {
                  dwMciFlags                 = MCI_VD_SET_DISPLAY | MCI_SET_OFF ;
                  ppiCurrent->fInfoDisplayOn = FALSE ;
                  fCheck                     = FALSE ;
                  }
               else
                  {
                  dwMciFlags                 = MCI_VD_SET_DISPLAY | MCI_SET_ON ;
                  ppiCurrent->fInfoDisplayOn = TRUE ;
                  fCheck                     = TRUE ;
                  }
               VdpSetDisplay ( ppiCurrent->wDeviceID, dwMciFlags ) ;
               WinCheckMenuItem ( hwndMenu, IDM_VDP_INFO_DISPLAY, fCheck ) ;
               break ;

            case IDM_VDP_AUDIO :
               // If audio is ON then set it OFF...
               // ...and vice versa...
               if ( ppiCurrent->fAudioAllOn )
                  {
                  dwMciFlags              = MCI_SET_AUDIO | MCI_SET_OFF ;
                  dwAudio                 = MCI_SET_AUDIO_ALL ;
                  ppiCurrent->fAudioAllOn = FALSE ;
                  fCheck                  = FALSE ;
                  }
               else
                  {
                  dwMciFlags              = MCI_SET_AUDIO | MCI_SET_ON ;
                  dwAudio                 = MCI_SET_AUDIO_ALL ;
                  ppiCurrent->fAudioAllOn = TRUE ;
                  fCheck                  = TRUE ;
                  }
               VdpSetAudio ( ppiCurrent->wDeviceID, dwMciFlags, dwAudio ) ;
               WinCheckMenuItem ( hwndMenu, IDM_VDP_AUDIO, fCheck ) ;
               break ;


            case IDM_VDP_STATUS:
               DisplayVideodiscStatus ( ppiCurrent->wDeviceID ) ;
               break ;


            case IDM_VDP_SPIN :
               // If the disc is spun up then spin it down...
               // ...and vice versa...
               if ( ppiCurrent->fSpunUp )
                  {
                  dwMciFlags              = MCI_SPIN_DOWN ;
                  ppiCurrent->fSpunUp     = FALSE ;
                  pszText                 = "Spi~n Up"  ;
                  }
               else
                  {
                  dwMciFlags              = MCI_SPIN_UP ;
                  ppiCurrent->fSpunUp     = TRUE ;
                  ppiCurrent->fDoorClosed = TRUE ; // Spin up will close the door also...
                  pszText                 = "Spi~n Down"  ;
                  }
               if ( VdpReady ( ppiCurrent->wDeviceID ) )
                  {
                  VdpSpin ( ppiCurrent->wDeviceID, dwMciFlags ) ;
                  }
               WinSetMenuItemText ( hwndMenu, IDM_VDP_SPIN, pszText ) ;
               break ;


            //***************************************
            //***   VIDEO INPUT Submenu...
            //***************************************

            case IDM_VID_INPUT_1:
            case IDM_VID_INPUT_2:
            case IDM_VID_INPUT_3:

               // Save the current value...
               OldVideoInput = paiMain->VideoInput ;
               NewVideoInput = ( usCommand - IDM_VID_INPUT_1 ) + 1 ;

               // If this is NOT a new video input...then take no further action
               if ( NewVideoInput == OldVideoInput )
                  {
                  ; // Nothing here...
                  }

               // Else this is a new video input
               else
                  {
                  // Change the video input...
                  dwReturn = OvlSetInput (
                                paiMain->wOverlayDeviceID,
                                NewVideoInput,
                                MCI_VIDEO_IN_CONNECTOR ) ;

                  // If the switch to a new video input was UNSUCCESSFUL...
                  if ( LOUSHORT ( dwReturn ) != MCIERR_SUCCESS )
                     {
                     ;  // Do nothing further...
                     }

                  // Else the switch was successful...
                  // ...We have many things to do...We are going to change
                  // the values associated w/ the new video input:
                  // source rectangle size, destination rectangle size,
                  // action mode (constant-aspect-ratio or adjust-source-area),
                  // window style, readout color...
                  else
                     {

                     // Update the checkmarks on the Input menu...
                     WinCheckMenuItem ( hwndMenu,
                           ( OldVideoInput - 1 ) + IDM_VID_INPUT_1, FALSE ) ;
                     paiMain->VideoInput = NewVideoInput ;
                     WinCheckMenuItem ( hwndMenu, usCommand, TRUE  ) ;

                     //--------------------------------------------------------
                     // This section switches all the characteristics
                     // associated with the video inputs 1, 2, and 3...
                     //
                     // Note the extra '{' '}' around this code section...
                     //
                     //--------------------------------------------------------
                     {
                     PVIDEOWINDOWINFO  pviOld, pviNew, pviCurrent ;
                     LONG              Index, Width, Height ;
                     LONG              OldStyle, NewStyle ;
                     LONG              FSMask, Active, Inactive ;
                     PSZ               *ppsz ;
                     PRECTL            prclFrame, prclDlg ;
                     HWND              hwndDlg ;
                     BOOL              fZoom, fEnableItems, fChangeBorder ;
                     BOOL              fShowChildWindow ;


                     //--------------------------------------------------------
                     // ==> 1. Initialize...
                     //--------------------------------------------------------
                     // Indicate that we are initializing....
                     paiMain->fInitializing = TRUE ;

                     // Show the wait pointer instead of the arrow pointer...
                     fShowWaitPointer = TRUE ;
                     WinSetPointer( HWND_DESKTOP,
                                    WinQuerySysPointer(HWND_DESKTOP,SPTR_WAIT,FALSE));

                     // Initialize...
                     pviCurrent = &paiMain->vwiInfo ;

                     //--------------------------------------------------------
                     // ==> 2. Save some current values to be used
                     //        when restoring later on...
                     //--------------------------------------------------------
                     // Save the frame position relative to the desktop...
                     pviCurrent->rclFrameOnDesktop = pviCurrent->rclFrame ;
                     WinMapWindowPoints ( hwndFrame, HWND_DESKTOP,
                             ( PPOINTL ) &pviCurrent->rclFrameOnDesktop, 2 ) ;

                     // If we are in maintain-client-aspect-ratio mode...
                     if ( pviCurrent->ActionMode
                                  == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
                        {
                        // Indicate no source control dlg active...
                        pviCurrent->fSrcControlDlgActive = FALSE ;
                        }

                     // Else we are in adjust-source-area mode...
                     else
                        {
                        // If the dialog is active then remember the
                        // appropriate information....
                        hwndDlg = paiMain->hwndSourceDlg ;
                        if ( hwndDlg )
                           {
                           pviCurrent->fSrcControlDlgActive = TRUE ;
                           prclDlg = &pviCurrent->rclSrcControlDlg ;
                           WinQueryWindowRect ( hwndDlg, prclDlg ) ;
                           WinMapWindowPoints ( hwndDlg, HWND_DESKTOP,
                                                ( PPOINTL ) prclDlg, 2 ) ;
                           }
                        // Else remember that the dialog is not active...
                        else
                           {
                           pviCurrent->fSrcControlDlgActive = FALSE ;
                           }
                        }

                     //--------------------------------------------------------
                     // ==> 3. Save the old control block and copy in
                     //        the new one...
                     //--------------------------------------------------------
                     // Save the old video window info block...
                     // ...and copy in the new video window info block...
                     pviOld      = &paiMain->aviVideoSources [ OldVideoInput - 1 ] ;
                     pviNew      = &paiMain->aviVideoSources [ NewVideoInput - 1 ] ;
                     *pviOld     = *pviCurrent ;
                     *pviCurrent = *pviNew ;

                     // Get the old and new style locally...for easy ref...
                     OldStyle   = pviOld->Style ;
                     NewStyle   = pviNew->Style ;

                     //--------------------------------------------------------
                     // ==> 4. Change the menu items and child window pos...
                     //--------------------------------------------------------
                     // Uncheck the old checks for STYLE and READOUT...
                     WinCheckMenuItem ( hwndMenu, pviOld->Style, FALSE ) ;
                     WinCheckMenuItem ( hwndMenu,
                                  pviOld->Readout + IDM_RDTCLR_NONE, FALSE ) ;

                     // Set the new checks for STYLE and READOUT...
                     if ( pviCurrent->ActionMode == VVK_AM_ADJUST_SOURCE_AREA )
                        {
                        WinCheckMenuItem ( hwndMenu, pviCurrent->Style, TRUE ) ;
                        }
                     WinCheckMenuItem ( hwndMenu,
                               pviCurrent->Readout + IDM_RDTCLR_NONE, TRUE ) ;

                     // Set the new child window pos...without affecting
                     // whether it is hidden or shown...
                     SetChildWindowPos ( hwndChild, pviCurrent ) ;

                     // Set the text of the menu items to new values...
                     // ...either the PAL window sizes or the NTSC window
                     // sizes...
                     for ( Index = 0, ppsz = pviCurrent->ppszWindowStrings ;
                           Index < 9 ; Index++, ppsz++ )
                        {
                        WinSetMenuItemText ( hwndMenu, IDM_VDW_CHILD + Index, *ppsz ) ;
                        }

                     //--------------------------------------------------------
                     // ==> 5. Handle transitions involved in going to
                     //        a different action mode...The "action mode"
                     //        is either the constant-aspect-ratio mode or
                     //        the adjust-source-area mode...
                     //--------------------------------------------------------
                     // Init...
                     fChangeBorder = FALSE ;

                     // If we are changing to a different action mode...
                     if ( pviCurrent->ActionMode != pviOld->ActionMode )
                        {
                        // Remember to change the border...
                        // ***Note: The sizing border is added/removed
                        // ***      under section 8 below depending on the
                        // ***      value of FSMask that we set here...
                        fChangeBorder = TRUE ;

                        // Set temporary variables based on the new mode...
                        if (pviCurrent->ActionMode==VVK_AM_ADJUST_SOURCE_AREA )
                           {
                           FSMask       = 0 ;
                           Active       = IDM_OVLY_ADJUST_SOURCE_AREA ;
                           Inactive     = IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO;
                           fEnableItems = TRUE ;
                           }
                        else
                           {
                           FSMask       = FS_SIZEBORDER ;
                           Active       = IDM_OVLY_MAINTAIN_CLIENT_ASPECT_RATIO;
                           Inactive     = IDM_OVLY_ADJUST_SOURCE_AREA ;
                           fEnableItems = FALSE ;
                           }

                        // Update menu check marks...
                        WinCheckMenuItem ( hwndMenu, Active, TRUE ) ;
                        WinCheckMenuItem ( hwndMenu, Inactive, FALSE ) ;

                        // Enable or disable video window menuitem...
                        WinEnableMenuItem (hwndMenu,IDM_VDW_MAIN,fEnableItems);
                        }

                     //--------------------------------------------------------
                     // ==> 6. "Take away" the source control dialog by
                     // ==>    hiding it or destroying it...
                     //--------------------------------------------------------
                     // If the source control dialog from the old
                     // input environment is active...
                     if ( paiMain->hwndSourceDlg != 0 )
                        {

                        // If the new environment also has the source
                        // control dialog active...then hide the dialog...
                        //
                        if ( pviCurrent->fSrcControlDlgActive )
                           {
                           WinSetWindowPos ( paiMain->hwndSourceDlg,
                                             0L,0,0,0,0, SWP_HIDE ) ;
                           }

                        // Else destroy the dialog...
                        else
                           {
                           WinDestroyWindow ( paiMain->hwndSourceDlg ) ;
                           }
                        }

                     //--------------------------------------------------------
                     // ==> 7. Hide the frame window...
                     //--------------------------------------------------------
                     WinSetWindowPos ( hwndFrame, 0L, 0, 0, 0, 0, SWP_HIDE ) ;

                     //--------------------------------------------------------
                     // ==> 8. Update the size of the frame...
                     //--------------------------------------------------------
                     // Add or remove sizing border...
                     if ( fChangeBorder )
                        {
                        WinSetWindowBits ( hwndFrame, QWL_STYLE,
                                           FSMask, FS_SIZEBORDER ) ;

                        // Tell the frame to update itself...
                        WinSendMsg ( hwndFrame, WM_UPDATEFRAME,
                                     ( MPARAM) FCF_SIZEBORDER,
                                     ( MPARAM ) 0 ) ;
                        }

                     // Change to the new frame position...
                     prclFrame = &pviCurrent->rclFrameOnDesktop ;
                     Width     = ( prclFrame->xRight - prclFrame->xLeft   ) ;
                     Height    = ( prclFrame->yTop   - prclFrame->yBottom ) ;
                     WinSetWindowPos ( hwndFrame, 0L,
                                       prclFrame->xLeft, prclFrame->yBottom,
                                       Width, Height,
                                       SWP_SIZE | SWP_MOVE ) ;

                     // Query the frame rectangle coordinates....
                     WinQueryWindowRect ( hwndFrame, &pviCurrent->rclFrame ) ;

                     // Change to new source rectangle...
                     OvlPutSrcRect ( paiMain->wOverlayDeviceID,
                                     &pviCurrent->rclCurSrc, ( HWND ) 0 ) ;

                     //--------------------------------------------------------
                     // ==> 9. Handle transitions involving the child
                     //         window...
                     //--------------------------------------------------------
                     // Init...
                     fShowChildWindow = FALSE ;

                     // If neither the previous style nor the new style
                     // is the child window...i.e. the client is the video
                     // window for the old input and the new input...
                     if (   OldStyle != IDM_VDW_CHILD
                         && NewStyle != IDM_VDW_CHILD )
                        {

                        // Make sure background color is transparent...
                        // ...On initialization, the background was set
                        // to CLR_BLK and may still be set so...
                        pviCurrent->clrClientBG = CLR_PINK ;

                        // Cause the client background to be filled on
                        // the next WM_PAINT...
                        fDoFill = TRUE ;
                        }

                     // Else if coming from a child window style
                     // AND going to a child window style....
                     else if (   OldStyle == IDM_VDW_CHILD
                              && NewStyle == IDM_VDW_CHILD )
                        {
                        ;  // Nothing to do here...
                        }

                     // Else if coming from a child window style to
                     // some other style...
                     else if ( OldStyle == IDM_VDW_CHILD )
                        {

                        // Re-enable the white readout menu selection...
                        WinEnableMenuItem (hwndMenu,IDM_RDTCLR_WHITE,TRUE) ;

                        // Set the new background color..
                        pviCurrent->clrClientBG = CLR_PINK ;

                        // Cause the client background to be filled on
                        // the next WM_PAINT...
                        fDoFill = TRUE ;

                        // Hide the child window...
                        WinSetWindowPos ( hwndChild, 0L,
                                          0, 0, 0, 0, SWP_HIDE ) ;

                        // Make the client window the new video window...
                        OvlSetVideoWindow ( paiMain->wOverlayDeviceID,
                                            hwndClient ) ;
                        }

                     // Else coming from some other window style
                     // to a child window style...
                     else
                        {

                        // Disable the white readout menu selection...
                        WinEnableMenuItem (hwndMenu,IDM_RDTCLR_WHITE,FALSE);

                        // Set the new background color..
                        pviCurrent->clrClientBG = CLR_WHITE ;

                        // Cause the client background to be filled on
                        // the next WM_PAINT...
                        //fDoFill = TRUE ;

                        // Make the child the video window...
                        OvlSetVideoWindow ( paiMain->wOverlayDeviceID,
                                            hwndChild ) ;

                        // Show the child window...
                        fShowChildWindow = TRUE ;
                        //WinSetWindowPos ( hwndChild, 0L,
                        //                  0, 0, 0, 0, SWP_SHOW ) ;
                        }

                     //--------------------------------------------------------
                     // ==> 10. Show the frame window...
                     //--------------------------------------------------------
                     WinSetWindowPos ( hwndFrame, 0L, 0, 0, 0, 0, SWP_SHOW ) ;

                     // Show the child window...
                     if ( fShowChildWindow )
                        {
                        WinSetWindowPos ( hwndChild, 0L,
                                          0, 0, 0, 0, SWP_SHOW ) ;
                        }

                     //--------------------------------------------------------
                     // ==> 11. Restore the source control dialog...
                     //
                     //--------------------------------------------------------
                     // If the new mode is constant-aspect-ratio...
                     if (pviCurrent->ActionMode
                                  == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
                        {
                        ; // Nothing to do here...
                        }

                     // Else the mode is adjust-source-area
                     else
                        {

                        // If the new environment uses the source control
                        // dialog...
                        if ( pviCurrent->fSrcControlDlgActive )
                           {

                           // If the source control dialog is NOT alive...
                           // ...then create it...
                           if ( paiMain->hwndSourceDlg == 0 )
                              {

                              // Put up the source control dialog...
                              // ...also remember/restore the zoom
                              // state, which gets forced to TRUE
                              // during the function call...
                              fZoom             = pviCurrent->fZoom ;
                              CreateSourceControlDialog (pviCurrent,FALSE) ;
                              pviCurrent->fZoom = fZoom ;

                              // If the zoom state is FALSE...i.e. if
                              // we are in STRETCH mode...
                              // ...then toggle to stretch mode...
                              if ( !pviCurrent->fZoom )
                                 {
                                 ToggleSourceControlDialog
                                    ( paiMain->hwndSourceDlg, pviCurrent ) ;
                                 }

                              // Now show it...
                              WinSetWindowPos ( paiMain->hwndSourceDlg,
                                                0L,0,0,0,0,SWP_SHOW ) ;
                              }
                           // Else adjust it...and show it...
                           else
                              {
                              // Get a copy of the (possibly new) handle...
                              hwndDlg = paiMain->hwndSourceDlg ;

                              // If the new source control dialog mode
                              // (i.e. ZOOM or STRETCH) is not the same
                              // as the old one....
                              // Then toggle to the new mode...
                              if ( pviCurrent->fZoom != pviOld->fZoom )
                                 {
                                 ToggleSourceControlDialog ( hwndDlg,
                                                             pviCurrent ) ;
                                 }

                              // Else the mode (ZOOM or STRETCH) is the
                              // same but all the values are different...
                              // ...update the source control dialog...
                              // ...it is currently hidden...
                              else
                                 {
                                 UpdateSourceArea ( hwndDlg, pviCurrent ) ;
                                 UpdatePanScrollBarCharacteristics
                                    ( hwndDlg, pviCurrent ) ;
                                 UpdateSourceZoomCharacteristics
                                    ( hwndDlg, pviCurrent ) ;
                                 UpdateHorzStretchCharacteristics
                                    ( hwndDlg, pviCurrent ) ;
                                 UpdateVertStretchCharacteristics
                                    ( hwndDlg, pviCurrent ) ;
                                 }

                              // Restore the previous position of the dlg...
                              // ...and show it...
                              prclDlg = &pviCurrent->rclSrcControlDlg ;
                              WinSetWindowPos ( hwndDlg,
                                                0L ,
                                                prclDlg->xLeft,
                                                prclDlg->yBottom,
                                                0, 0, SWP_MOVE | SWP_SHOW ) ;
                              }
                           }
                        }

                     //--------------------------------------------------------
                     // ==> 12. Done...Clean up...
                     //--------------------------------------------------------
                     // Stop showing the wait pointer...
                     fShowWaitPointer = FALSE ;

                     // Indicate that we finished initializing....
                     paiMain->fInitializing = FALSE ;

                     // Update the window...
                     WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
                     WinUpdateWindow   ( hwndClient ) ;
                     }  // End of the embedded section...
                     }     // End of the big ELSE statement...
                  }
               break;


            //***************************************
            //*** VIDEO WINDOW submenu...
            //***************************************

            case IDM_VDW_CHILD              :
            case IDM_VDW_CLIENT_FULL_EXTENT :
            case IDM_VDW_CLIENT_04TH_EXTENT :
            case IDM_VDW_CLIENT_16TH_EXTENT :
            case IDM_VDW_CLIENT_64TH_EXTENT :
            case IDM_VDW_CLIENT_ARBITRARY_1 :
            case IDM_VDW_CLIENT_ARBITRARY_2 :
            case IDM_VDW_CLIENT_ARBITRARY_3 :
               ChangeVideoWindowStyle ( &paiMain->vwiInfo,
                                        paiMain->vwiInfo.Style,
                                        usCommand ) ;
               break;



            //***************************************
            //*** READOUTS Submenu...
            //*************************************** */

            case IDM_RDTCLR_NONE  :
            case IDM_RDTCLR_WHITE :
            case IDM_RDTCLR_YELLOW:
            case IDM_RDTCLR_CYAN  :
            case IDM_RDTCLR_BLACK :

               // Reset the saved value...throw it away...
               pvi->SaveReadout = VVK_RO_NONE ;

               // Save the old value...
               OldReadout = pvi->Readout ;

               // Set the new value
               pvi->Readout = VVK_RO_NONE + ( usCommand - IDM_RDTCLR_NONE ) ;

               // Set the new foreground color...a value of "none" does
               // not change the foreground color...
               MapReadoutToClr ( pvi->Readout, &pvi->clrClientFG ) ;

               // Uncheck the old menu item...and check the new one...
               WinCheckMenuItem ( hwndMenu, IDM_RDTCLR_NONE + OldReadout,
                                  FALSE ) ;
               WinCheckMenuItem ( hwndMenu, usCommand, TRUE ) ;

               // Re-paint the client...
               WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
               WinUpdateWindow ( hwndClient ) ;
               break;

            //*******************************
            //*** HELP SUBMENU OPTIONS
            //*******************************

            case IDM_HELPITEM:

                WinDlgBox(HWND_DESKTOP, hwnd,
                        DevHelpInfoDlgProc,
                        (HMODULE) 0, ID_HELP_INFO_DLG,
                        0);
               break;

            case IDM_ABOUT:

                  WinDlgBox( HWND_DESKTOP,
                        HWND_DESKTOP,
                        ( PFNWP )ProdInfoDlgProc,
                        ( HMODULE ) 0,
                        ID_DLG_PRODINFO,
                        NULL );
               break;

            default:
               return WinDefWindowProc( hwnd, msg, mp1, mp2 );
            }
         break;


      case MM_MCIPASSDEVICE:
         {
         WORD     wDeviceID ;
         USHORT   usPassCommand ;
         SZ       szText [ 60 ] ;

         // Get the device id and the passdevice-command...
         wDeviceID     = SHORT1FROMMP ( mp1 ) ;
         usPassCommand = SHORT1FROMMP ( mp2 );

         // If overlay device....
         if ( wDeviceID == paiMain->wOverlayDeviceID )
            {
            // Overlay device processing would go here...
            // ...but the overlay device code for the video
            // window will automatically try to acquire the
            // device when it receives a WM_ACTIVATE...

            switch (usPassCommand)
               {
               case MCI_LOSING_USE:
                  paiMain->fOwnDevice = FALSE ;

                  sprintf ( szText, "Losing use of Overlay\n"
                                    "Overlay Device  %d", wDeviceID ) ;

                  // ***UNCOMMENT the next line if you want to see the message...
                  //DisplayPassDeviceMessage ( szText ) ;
                  break;

               case MCI_GAINING_USE:
                  paiMain->fOwnDevice = TRUE ;

                  sprintf ( szText, "Gaining use of Overlay\n"
                                     "Overlay Device  %d", wDeviceID ) ;

                  // ***UNCOMMENT the next line if you want to see the message...
                  //DisplayPassDeviceMessage ( szText ) ;


                  // Check overlay device...
                  if ( !paiMain->vwiInfo.fInitialized )
                     {
                     InitializeVideoWindowInfo ( &paiMain->vwiInfo, paiMain->VideoInput ) ;
                     paiMain->vwiInfo.fInitialized = TRUE ;
                     }
                  break;

               default:
                  // This should not occur...
                  //sprintf ( szText, "Unknown Message, Overlay\n"
                  //                   "Device  %d\n"
                  //                   "Message %d",
                  //                   usPassCommand, wDeviceID ) ;
                  //DisplayPassDeviceMessage ( szText ) ;
                  break;
               }
            }

         // Else if videodisc player device...
         else if ( wDeviceID == ppiCurrent->wDeviceID )
            {

            switch (usPassCommand)
               {
               case MCI_LOSING_USE:
                  ppiCurrent->fOwnDevice = FALSE ;

                  sprintf ( szText, "Losing use of Videodisc\n"
                                    "Player Device  %d", wDeviceID ) ;

                  // ***UNCOMMENT the next line if you want to see the message...
                  //DisplayPassDeviceMessage ( szText ) ;
                  break;

               case MCI_GAINING_USE:
                  ppiCurrent->fOwnDevice = TRUE ;
                  sprintf ( szText, "Gaining use of Videodisc\n"
                                     "Player Device  %d", wDeviceID ) ;

                  // ***UNCOMMENT the next line if you want to see the message...
                  //DisplayPassDeviceMessage ( szText ) ;
                  break;

               default:
                  // This should not occur...
                  //sprintf ( szText, "Unknown Message, Videodisc\n"
                  //                   "Player Device  %d\n"
                  //                   "Message %d",
                  //                   usPassCommand, wDeviceID ) ;
                  //DisplayPassDeviceMessage ( szText ) ;
                  break;
               }
            }
         }
         break;


      case WM_MINMAXFRAME:

         // Get access to SWP structure...
         pswp = ( PSWP ) mp1 ;

         // If minimizing...
         if ( pswp->fl & SWP_MINIMIZE )
            {
            // Set "minimized flag so our WM_SIZE/WM_MOVE processing will
            // not interfere with the appearance of the icon...
            paiMain->fMinimized = TRUE ;
            }

         // Else if restoring and was minimized...then enable normal
         // sizing operation...
         else if ( pswp->fl & SWP_RESTORE && paiMain->fMinimized )
            {
            paiMain->fMinimized = FALSE ;
            }

         // Else if maximizing...
         else if ( pswp->fl & SWP_MAXIMIZE )
            {
            // If in maintain-client-aspect-ratio mode...
            if ( paiMain->vwiInfo.ActionMode == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
               {

               // Display a hopefully helpful message...
               WinMessageBox ( HWND_DESKTOP, HWND_DESKTOP, (PCH) szFullScreenMessage,
                  ( PCH ) "Full Screen Client Area", ID_MESSAGEBOX,
                  MB_OK | MB_SYSTEMMODAL | MB_ICONEXCLAMATION );
               }

            // Else in adjust-source-area mode...
            else
               {
               // If the source area is less than the destination area
               // then we need to increase the size of the source area
               // to match the destination area...
               ;     // Need to add code here...Exercise for user!
               }
            }
         return (MRESULT)FALSE;


      case WM_ERASEBACKGROUND:

         // Paint the window background in the current client color...
         //
         // Notice that only the area that needs updating gets updated...
         // ...compare WM_PAINT, where displaying by the overlay device
         // sometimes invalidates the entire window rectangle...
         WinFillRect((HPS)mp1, (PRECTL)mp2, paiMain->vwiInfo.clrClientBG ) ;
         return (MRESULT)FALSE ;


      case WM_MOVE:

         // Get pointer to current video window information...
         pvi = &paiMain->vwiInfo ;

         // Store frame position...
         WinQueryWindowRect ( hwndFrame, &pvi->rclFrame ) ;

         // Update the absolute position of the client area...
         pvi->rclClientOnDesktop = pvi->rclClient ;
         WinMapWindowPoints ( hwndClient, HWND_DESKTOP,
                             ( PPOINTL ) &pvi->rclClientOnDesktop, 2 ) ;

         // If the child of the client is the video window, and the readouts
         // are active, then force a paint so that the readouts will be
         // updated.
         if (    pvi->Style == IDM_VDW_CHILD
              && pvi->Readout != VVK_RO_NONE )
            {
            WinInvalidateRect ( hwndClient, NULL, FALSE ) ;
            WinUpdateWindow   ( hwndClient ) ;
            }
         break ;


      case WM_SIZE:

         // If minimizing then do not do the activity below...it interferes
         // with the appearance of the icon...
         if ( paiMain->fMinimized )
            {
            break ;
            }

         // If this WM_SIZE call is due to the WinSetWindowPos() call
         // that sets the frame size....then ignore this call...
         if ( paiMain->fSettingFramePos )
            {
            break ;
            }

         // Get pointer to current video window information...
         pvi = &paiMain->vwiInfo ;

         // Save the old client size in client coordinates and desktop coordinates...
         rclOldClient          = pvi->rclClient          ;
         rclOldClientOnDesktop = pvi->rclClientOnDesktop ;

         // Store the new client rectangle...
         prcl = &pvi->rclClient ;
         prcl->xLeft   = 0 ;
         prcl->xRight  = ( SHORT1FROMMP ( mp2 ) - 1 ) ;
         prcl->yBottom = 0 ;
         prcl->yTop    = ( SHORT2FROMMP ( mp2 ) - 1 ) ;

         // Also store the new client rectangle in desktop coordinates...
         pvi->rclClientOnDesktop = pvi->rclClient ;
         WinMapWindowPoints ( hwndClient, HWND_DESKTOP,
                             ( PPOINTL ) &pvi->rclClientOnDesktop, 2 ) ;

         // If we are in maintain-client-aspect-ratio mode...
         if (   !paiMain->fInitializing
             && pvi->ActionMode == VVK_AM_MAINTAIN_CLIENT_ASPECT_RATIO )
            {

            // Map the new rectangle to the nearest proportionally sized rectangle...
            //
            // The choice for a min rectangle is arbitrary...we are using
            // the size of our smallest selectable rectangle as the min...
            // ...the actual min destination rectangle imposed by the
            // hardware is 4 pels x 1 pel...
            //
            prclMin   = &pvi->rclMinClient ;
            MinWidth  = ( prclMin->xRight - prclMin->xLeft   ) + 1 ;
            MinHeight = ( prclMin->yTop   - prclMin->yBottom ) + 1 ;
            prclMax   = &pvi->rclCurSrc ;
            MaxWidth  = ( prclMax->xRight - prclMax->xLeft   ) + 1 ;
            MaxHeight = ( prclMax->yTop   - prclMax->yBottom ) + 1 ;
            rclNew    = pvi->rclClientOnDesktop ;
            fSuccess  = MapRectToProRect ( &rclOldClientOnDesktop,
                                           &rclNew, &rclResult,
                                           pvi->SrcAspectX,
                                           pvi->SrcAspectY,
                                           MinWidth, MinHeight,
                                           MaxWidth, MaxHeight ) ;

            // If successful...
            if ( fSuccess )
               {
               // Store new client size...in client coords and desktop coords...
               pvi->rclClientOnDesktop = rclResult ;
               pvi->rclClient          = rclResult ;
               WinMapWindowPoints ( HWND_DESKTOP, hwndClient,
                             ( PPOINTL ) &pvi->rclClient, 2 ) ;

               // Calculate the equivalent frame rectangle...still in
               // world coordinates...
               rclNewFrame = pvi->rclClientOnDesktop ;
               WinCalcFrameRect ( hwndFrame, &rclNewFrame, FALSE ) ;

               // Set the frame to the newly calculated position...
               // NOTE: The call below will result in another WM_SIZE
               // call to this window procedure....We set
               // paiMain->fSettingFramePos to TRUE to indicate that
               // we can ignore that particular WM_SIZE call...
               paiMain->fSettingFramePos = TRUE ;
               prcl = &rclNewFrame ;
               WinSetWindowPos ( hwndFrame, 0L ,
                              prcl->xLeft,
                              prcl->yBottom,
                              ( prcl->xRight - prcl->xLeft   ) + 1,
                              ( prcl->yTop   - prcl->yBottom ) + 1,
                               SWP_MOVE | SWP_SIZE ) ;
               paiMain->fSettingFramePos = FALSE ;

               // Query and store the frame position....
               // Get the frame rectangle coordinates....
               WinQueryWindowRect ( hwndFrame, &pvi->rclFrame ) ;
               }
            }
         break ;


      case WM_PAINT:
         {
         // If the client is a video window...
         //    1) We will see MORE WM_PAINTS than we would normally
         //       expect.
         //    2) We will see the ENTIRE WINDOW invalidated when
         //       we otherwise wd not expect it.  For example,
         //       a WM_PAINT following a WM_MOVE...
         //
         HPS    hps ;
         RECTL  rcl ;

         // Display the readout values...
         hps = WinBeginPaint ( hwnd, (HPS)NULL, &rcl ) ;
         if ( !paiMain->fInitializing )
            {
            // Under some conditions fill the entire area...
            if ( fDoFill )
               {
               fDoFill = FALSE ;
               WinFillRect ( hps, &rcl, paiMain->vwiInfo.clrClientBG ) ;
               }

            // If the readouts are active then display them...
            DisplayValuesInClientArea ( hps, &paiMain->vwiInfo ) ;
            }

         WinEndPaint ( hps ) ;
         }
         break;


      case WM_CONTROLPOINTER:
         if ( fShowWaitPointer )
            {
            return ( MRESULT )  WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE ) ;
            }
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );


      case WM_MOUSEMOVE:
         if ( fShowWaitPointer )
            {
            HPOINTER hptrWait ;

            hptrWait = WinQuerySysPointer( HWND_DESKTOP, SPTR_WAIT, FALSE );
            WinSetPointer( HWND_DESKTOP, hptrWait );
            return ( MRESULT ) TRUE ;
            }
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );


      case WM_CLOSE:

         // Close any open dialog windows...
         if ( paiMain->hwndSourceDlg )
            {
            WinDestroyWindow ( paiMain->hwndSourceDlg ) ;
            }

         // Cause termination...
         WinPostMsg( hwnd, WM_QUIT, NULL, NULL);
         break;

      default:
         // Must have WinDefWindowProc in your window procedure
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }

//-----------------------------------------------------------------------------
// Name:          SeekDlgProc
//
// Description:   Processes messages for Seek dialog
//
// Concepts:      Gets a numerical input and returns.
//
// Parameters:    Std dialog proc parms
//
// Returns:       Std dialog proc returns
//
//-----------------------------------------------------------------------------
MRESULT EXPENTRY SeekDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
   {
   USHORT usSeekFrame = 1;
   BOOL fSuccess ;

   switch(msg)
      {
      case WM_INITDLG :

         // Limit text input to 5 characters...
         WinSendDlgItemMsg ( hwnd, ID_SEEK_ENTRY_FIELD, EM_SETTEXTLIMIT,
                             MPFROM2SHORT ( 5, 0), NULL ) ;

         // Show the current seek value...
         WinSetDlgItemShort ( hwnd, ID_SEEK_ENTRY_FIELD,
                              ( USHORT ) paiMain->SeekFrame,
                              FALSE ) ;

         return ( MRESULT ) 0 ;

      case WM_COMMAND :
         // Switch on the command...
         switch ( SHORT1FROMMP ( mp1 ) )
            {

            // User selected "OK"...so we process the contents of the
            // entry field...and close if they are valid...
            case DID_OK:

               // Init...
               fSuccess = FALSE ;

               // If the entry field changed...
               if ( WinSendDlgItemMsg ( hwnd, ID_SEEK_ENTRY_FIELD,
                                        EM_QUERYCHANGED,
                                        ( MPARAM ) 0, ( MPARAM) 0 ) )
                  {
                  fSuccess = WinQueryDlgItemShort (
                             hwnd,
                             ID_SEEK_ENTRY_FIELD,
                             ( PSHORT ) &usSeekFrame,
                             FALSE ) ;
                  if ( usSeekFrame <= 0 || usSeekFrame > 54000 )
                     {
                     fSuccess = FALSE ;
                     }
                  }
               if ( fSuccess )
                  {
                  paiMain->SeekFrame = ( LONG ) usSeekFrame ;
                  WinDismissDlg ( hwnd, TRUE ) ;
                  }
               break;


            // User selected "CANCEL"...so we close the dialog...without
            // saving the value...
            case DID_CANCEL:
               WinDismissDlg ( hwnd, FALSE ) ;
               break;
            }
         return ( MRESULT ) 0 ;


      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
       }
   return (MRESULT)0;
   }


//-----------------------------------------------------------------------------
// Name:          ProdInfoDlgProc
//
// Description:   Processes all messages sent to the Product Information dialog box.
//
// Concepts:      Gets a numerical input and returns.
//
// Parameters:    Std dialog proc parms
//
// Returns:       Std dialog proc returns
//
//-----------------------------------------------------------------------------
MRESULT EXPENTRY ProdInfoDlgProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
   {
   switch(msg)
      {
      case WM_INITDLG:
         // If initializing...Start a timer...when it pops we will return...
         if ( paiMain->fInitializing )
            {
            WinStartTimer ( hab, hwnd, ID_TIMER, 2000 ) ;
            }
         break;

      case WM_COMMAND:
         // No matter what the command, close the dialog...
         WinDismissDlg ( hwnd, TRUE ) ;
         break;

      case WM_TIMER:
         // Free the timer and close the dialog...
         WinStopTimer ( hab, hwnd, ID_TIMER ) ;
         WinDismissDlg ( hwnd, TRUE ) ;
         break;

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
       }
   return (MRESULT)0;
   }


//-----------------------------------------------------------------------------
// Name:        DevHelpInfoDlgProc()
//
// Description: This dialog procedure handles the dialog help window.
//              It displays a window with a list box where the help information
//              text is displayed. The list box is scrollable. The dialog box
//              has a system menu only. Thus, the user must close it from the
//              system menu.
//
// Concepts:
//                Gets a numerical input and returns.
//
// Parameters:    Std dialog proc parms
//
// Returns:       Std dialog proc returns
//
//-----------------------------------------------------------------------------
MRESULT EXPENTRY DevHelpInfoDlgProc( HWND hwndDlg, ULONG msg, MPARAM mp1, MPARAM mp2 )
   {
   int      count;
   COLOR    TransColor = CLR_PALEGRAY;
   CHAR     szTempStr[100];
   LONG     TitleBarHeight, DlgBorderHeight, DlgBorderWidth ;
   SWP      swpListbox, swpDlg ;

   switch ( msg )
      {
      case WM_INITDLG:

         // Show SPTR_WAIT pointer...
         fShowWaitPointer = TRUE ;
         WinSetPointer( HWND_DESKTOP,
                        WinQuerySysPointer(HWND_DESKTOP,SPTR_WAIT,FALSE));

         // Create help dialog and insert the lines of text...
         for ( count=0 ; count<NUM_HELP_LINES ; count++ )
            {
            WinLoadString ( hab,
                            (HMODULE) 0,
                            (ULONG)ID_HELP_TEXT_LINE_01 + count,
                            (LONG)100,
                            szTempStr);

            WinSendDlgItemMsg( hwndDlg,
                              ID_HELP_LISTBOX,
                              LM_INSERTITEM,
                              MPFROMSHORT(LIT_END),
                              MPFROMP (szTempStr));
            WinSetPresParam (
               hwndDlg,
               PP_BACKGROUNDCOLORINDEX,
               (ULONG) sizeof (TransColor),
               (PVOID) &TransColor);
            }

         // Get the size of the list box and the dialog window...
         WinQueryWindowPos ( WinWindowFromID ( hwndDlg, ID_HELP_LISTBOX),
                             &swpListbox );
         WinQueryWindowPos ( hwndDlg, &swpDlg );

         // Get the heights of the title bar and dlg border...
         TitleBarHeight  = WinQuerySysValue ( HWND_DESKTOP, SV_CYTITLEBAR ) ;
         DlgBorderHeight = WinQuerySysValue ( HWND_DESKTOP, SV_CYDLGFRAME ) ;
         DlgBorderWidth  = WinQuerySysValue ( HWND_DESKTOP, SV_CXDLGFRAME ) ;

         // Make the height of the dialog equal to the height of the
         // listbox PLUS (height of titlebar + 2 * height of dlgborder)...
         swpDlg.cy = swpListbox.cy + ( TitleBarHeight + (2 * DlgBorderHeight));

         // Make the width of the dialog equal to the width of the listbox
         // PLUS 2 dialog borders....
         swpDlg.cx = swpListbox.cx + ( 2 * DlgBorderWidth ) ;

         // Set the new dialog window size...
         WinSetWindowPos ( hwndDlg, (ULONG)0,
                           0,0,swpDlg.cx, swpDlg.cy, SWP_SIZE );

         // Move the listbox just inside the dialog border
         WinSetWindowPos ( WinWindowFromID ( hwndDlg, ID_HELP_LISTBOX),
                           (ULONG)0,
                           DlgBorderWidth, DlgBorderHeight, 0, 0, SWP_MOVE ) ;

         // Restore system pointer...
         fShowWaitPointer = FALSE ;
         break;


      default:
         return WinDefDlgProc( hwndDlg, msg, mp1, mp2 );
      }
   return (MRESULT) FALSE;
   }


//-----------------------------------------------------------------------------
// Name:          DisplayMessage
//
// Description:   Displays error message in a message box.
//
// Concepts:      Common message handling
//
// Parameters:    str - displays this string in the message box
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID DisplayMessage ( PCH str )
   {
   WinMessageBox(HWND_DESKTOP, HWND_DESKTOP, (PCH) str,
      (PCH) "Error Message Box For VIDVIEW Sample", ID_MESSAGEBOX,
      MB_OK | MB_APPLMODAL | MB_MOVEABLE );
   }

//-----------------------------------------------------------------------------
// Name:          DisplayPassDeviceMessage()
//
// Description:   Displays "gaining device" and "losing device messages".
//
// Concepts:      Illustrates device sharing by displaying message when
//                gaining/losing device occurs.
//
// Parameters:    str - string to be displayed
//
// Returns:       VOID
//
//-----------------------------------------------------------------------------
VOID DisplayPassDeviceMessage ( PCH str )
   {
   WinMessageBox ( HWND_DESKTOP, HWND_DESKTOP, ( PCH)  str,
                   ( PCH ) "MM_MCIPASSDEVICE for VIDVIEW",
                   ID_MESSAGEBOX,
                    MB_OK | MB_APPLMODAL | MB_MOVEABLE );
   }


//*****************************************************************************
//***************************************************************************** */
//***
//*** MCI Calls Section
//***    -- General calls...
//***    -- Videodisc calls...
//***    -- Overlay calls ===> See VIDVIEW2.C
//***
//*****************************************************************************
//*****************************************************************************

//-----------------------------------------------------------------------------
//
// Name:          OpenDevice
//
// Description:   Opens an MCI device...
//
// Concepts:      Opens w/ specified device name and alias
//
//                If you are having trouble opening check your use
//                of MCI_OPEN_SHAREABLE.
//
// Parameters:    lpstrDevName - name
//
//                lpstrDevAlias - alias
//
//                dwMciFlags - MCI_* flags
//
//                pwDeviceID - returns device id from open
//
// Returns:       FALSE if successful, TRUE otherwise...
//
//-----------------------------------------------------------------------------
DWORD OpenDevice ( LPSTR lpstrDevName, LPSTR lpstrDevAlias, DWORD dwMciFlags, WORD *pwDeviceID)
   {
   MCI_VID_OPEN_PARMS  mciOpenParms;
   WORD    wUserParm = 0;
   DWORD   dwReturnCode;

   // NOTE: *** Specify a return address for dwCallback if you want to see
   //       *** MM_MCIPASSDEVICE and/or MM_MCINOTIFY notify messages.
   //
   mciOpenParms.dwCallback = (DWORD)hwndNotify;
   mciOpenParms.wDeviceID = 0;
   mciOpenParms.lpstrDeviceType = lpstrDevName;
   mciOpenParms.lpstrElementName = 0;

   mciOpenParms.lpstrAlias = lpstrDevAlias;
   mciOpenParms.hwndParent = 0;

   dwReturnCode = mciSendCommand(0, MCI_OPEN,
                  MCI_OPEN_ALIAS | dwMciFlags,
                  (DWORD)&mciOpenParms,
                  wUserParm);

   if (LOUSHORT(dwReturnCode) == MCIERR_SUCCESS)
      {
      *pwDeviceID = mciOpenParms.wDeviceID;
      return (FALSE);
      }

   return (TRUE);
   }

//-----------------------------------------------------------------------------
//
// Name:          CloseDevice
//
// Description:   Closes an open MCI device...
//
// Concepts:      Uses generic parm structure.
//
// Parameters:    wDeviceID - id of dev to be closed
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD CloseDevice (WORD wDeviceID)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturnCode;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;

   dwReturnCode = mciSendCommand(wDeviceID, MCI_CLOSE,
                  MCI_WAIT,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }


//-----------------------------------------------------------------------------
//
// Name:          AcquireDevice()
//
// Description:   Acquires an MCI device
//
// Concepts:      If the device is opened shareable, another application
//                can acquire it from you, and vice versa.
//
//                Acquiring is an intentional deed; losing may be
//                without your knowledge (unless you specify a callback
//                address on the open, and detect MCI_PASSDEVICE messages).
//
// Parameters:    wDeviceID   - to acquire
//                dwExclusive - exclusive acquire
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD AcquireDevice (WORD wDeviceID, DWORD dwExclusive)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturnCode;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;
   dwReturnCode = mciSendCommand(wDeviceID, MCI_ACQUIREDEVICE,
                  MCI_WAIT | dwExclusive,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }

//-----------------------------------------------------------------------------
//
// Name:          ReleaseDevice()
//
// Description:   Releases an MCI device
//
// Concepts:      Opposite of acquire, see note to AcquireDevice()
//
// Parameters:    wDeviceID - the device id
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD ReleaseDevice (WORD wDeviceID)
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   WORD    wUserParm = 0;
   DWORD   dwReturnCode;

   mciGenericParms.dwCallback = (DWORD)hwndNotify;
   dwReturnCode = mciSendCommand(wDeviceID, MCI_RELEASEDEVICE,
                  MCI_WAIT,
                  (DWORD)&mciGenericParms,
                  wUserParm);

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }

//-----------------------------------------------------------------------------
//
// Name:          VdpPlay
//
// Description:   Plays the videodisc player forward or backward.
//
// Concepts:      You can specify EITHER a starting and ending location
//                OR a direction (forward or backward)...
//
// Parameters:    wDeviceID - the device
//                fFoward - TRUE if forward, FALSE is backward
//                dwFromFrame - starting location
//                dwToFrame   - ending location
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD VdpPlay ( WORD wDeviceID, BOOL fForward, DWORD dwFromFrame,  DWORD dwToFrame )
   {
   MCI_VD_PLAY_PARMS mciVdPlayParms ;

   DWORD  dwMciFlags = 0 ;
   DWORD   dwReturnCode;

   // Send an MCI_PLAY command...
   memset ( &mciVdPlayParms, 0x00, sizeof ( MCI_VD_PLAY_PARMS ) ) ;
   mciVdPlayParms.dwFrom     = dwFromFrame ;
   mciVdPlayParms.dwTo       = dwToFrame ;
   mciVdPlayParms.dwFactor   = 0 ;
   if ( !fForward )
      {
      dwMciFlags = MCI_VD_PLAY_REVERSE ;
      }
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_PLAY,
                  dwMciFlags,
                  ( DWORD )&mciVdPlayParms,
                  0 ) ;

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }

//-----------------------------------------------------------------------------
//
// Name:          VdpSeek
//
// Description:   Seeks the videodisc player forward or backward.
//
// Concepts:      Specify a location, or a flag such as MCI_TO_START,
//                MCI_TO_END, MCI_VD_SEEK_REVERSE
//
// Parameters:    wDeviceID - the device
//
//                SeekFrame - the destination
//
//                dwMciFlags - any additional flags
//
//                hwndCallback - callback address
//
// Returns:       Return code from MciSendCommand()
//
//-----------------------------------------------------------------------------
DWORD VdpSeek ( WORD wDeviceID, LONG SeekFrame, DWORD dwMciFlags,
                HWND hwndCallback, WORD wUserParm )
   {
   MCI_SEEK_PARMS mciSeekParms ;

   DWORD  dwReturnCode;

   // Send an MCI_SEEK command...
   memset ( &mciSeekParms, 0x00, sizeof ( MCI_SEEK_PARMS ) ) ;
   mciSeekParms.dwCallback = ( DWORD ) hwndCallback ;
   mciSeekParms.dwTo       = SeekFrame ;
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_SEEK,
                  dwMciFlags | MCI_TO,
                  ( DWORD )&mciSeekParms,
                  wUserParm ) ;

   return dwReturnCode ;
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpStop
//
// Description:   Stops the player.
//
// Concepts:      Works whether player is playing, seeking, or already stopped.
//
//                Note: MCP/2 videodisc player drivers do NOT automatically
//                turn off the video when the player is stopped.
//
// Parameters:    wDeviceID - the device
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD VdpStop ( WORD wDeviceID )
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   DWORD   dwReturnCode;

   // Send an MCI_STOP command...
   memset ( &mciGenericParms, 0x00, sizeof ( MCI_GENERIC_PARMS ) ) ;
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_STOP,
                               0,
                               ( DWORD ) &mciGenericParms,
                               0 );

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpSpin
//
// Description:   Spins the player up or down.
//
// Concepts:      This is a time-consuming process, so you probably want
//                to specify MCI_NOTIFY and a callback address to be
//                notified when the operation is done.  Or the app
//                can monitor the player status usng MCI_VDP_GET_MODE
//                until the mode becomes MCI_VDP_MODE_PARKED (spun down)
//                or MCI_VDP_MODE_STILL (spun up and not moving).
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - such as MCI_NOTIFY
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD VdpSpin ( WORD wDeviceID, DWORD dwMciFlags )
   {
   MCI_GENERIC_PARMS  mciGenericParms;
   DWORD   dwReturnCode;

   // Send an MCI_SPIN_UP or MCI_SPIN_DOWN command...
   memset ( &mciGenericParms, 0x00, sizeof ( MCI_GENERIC_PARMS ) ) ;
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_SPIN,
                               dwMciFlags,
                               ( DWORD )&mciGenericParms,
                               0 );

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpSetDisplay
//
// Description:   Sets player info display on or off
//
// Concepts:      This is a frame-number display done by the player itself.
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - This is a very quick operation; you can very
//                             probably get by w/out specifying MCI_NOTIFY
//                             or MCI_WAIT.
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD VdpSetDisplay ( WORD wDeviceID, DWORD dwMciFlags )
   {
   MCI_VD_SET_PARMS  mciVdSetParms ;
   DWORD   dwReturnCode ;

   // Send an MCI_SET command to switch the information display on or off...
   memset ( &mciVdSetParms, 0x00, sizeof ( MCI_VD_SET_PARMS ) ) ;
   dwReturnCode = mciSendCommand( wDeviceID, MCI_SET,
                              dwMciFlags,
                              ( DWORD )&mciVdSetParms,
                              0 );

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpReady()
//
// Description:   Simple error-check-and-recovery function.
//
// Concepts:      Here we treat the player as if it were a printer;
//                before "printing", we ask the "printer" if it is ready;
//                if not, then we prompt the user to check the device.
//
// Parameters:    wDeviceID - the device
//
// Returns:       TRUE if successful, else FALSE
//
//-----------------------------------------------------------------------------
BOOL VdpReady  ( WORD wDeviceID )
   {
   DWORD dwReturnCode, dwReturn ;

   // If we do not own the device then acquire it...
   // NOTE: This section is overkill, because we specifically re-acquire
   //       the device in the WM_ACTIVATE call....
   if ( !ppiCurrent->fOwnDevice )
      {
      AcquireDevice ( wDeviceID, 0 ) ;
      }

   // Query the mode...a very quick call...the device constantly monitors
   // the mode...
   dwReturnCode = VdpGetStatus ( wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_MODE,
                                       &dwReturn ) ;
   // If successful call...
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {

      // If the player is not ready then display an operator intervention
      // message...
      if ( dwReturn == MCI_MODE_NOT_READY )
         {
         DisplayMessage ("The videodisc player is not ready.  Please "
                         "verify that the player is powered on and that "
                         "a videodisc is loaded in it right side up; "
                         "then retry the operation." ) ;
         return FALSE ;
         }
      }
   return TRUE ;
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpGetStatus()
//
// Description:   Returns status for specified videodisc player device item.
//
//                ***SEE DisplayVideodiscStatus() for examples of how to
//                interpret data returned.***
//
// Concepts:      The overlay device constantly monitors the device, so
//                it can process a status call IMMEDIATELY w/out querying
//                the device.
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - MCI_* flags
//
//                dwItem - which MCI_STATUS option
//
//                *pdwReturn - pointer to status value returned.
//                             ***CALLER MUST ASSURE that this points
//                             to a valid doubleword.***
//
// Returns:       Return code from mciSendCommand()
//
//-----------------------------------------------------------------------------
DWORD VdpGetStatus ( WORD wDeviceID, DWORD dwMciFlags, DWORD dwItem,
                           DWORD *pdwReturn )
   {
   MCI_STATUS_PARMS  mciStatusParms ;
   DWORD             dwReturnCode ;

   // Set up the parm structure...
   memset ( &mciStatusParms, 0x00, sizeof ( MCI_STATUS_PARMS ) ) ;
   mciStatusParms.dwItem     = dwItem ;
   mciStatusParms.dwValue    = 0      ;

   // Do the MCI_STATUS call...
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_STATUS,
                                dwMciFlags,
                                ( DWORD ) &mciStatusParms,
                                0 );

   // Set the return value...and return...
   *pdwReturn = mciStatusParms.dwReturn ;
   return dwReturnCode ;
   }


//-----------------------------------------------------------------------------
//
// Name:          VdpGetCustomStatus()
//
// Description:   Returns status videodisc status using custom command
//                MCI_VDP_GET_MODE.
//
//                ***Note***: Be sure to include "lvdsys.h" when you use this...
//
// Concepts:      The MCI_STATUS command w/ the MCI_STATUS_MODE option
//                returns 6 possible player states:  stopped, playing,
//                not ready, paused, playing, seeking, or stopped.
//
//                The MCI_VDP_GET_MODE command returns the same player states,
//                PLUS other states that MCI_STATUS_MODE treats as
//                equivalent to "stopped".  The additional states are:
//                still (stopped, but video showing), parked, no_disc,
//                door open.
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - MCI_* flags
//
//                *pdwReturn - pointer to status value returned.
//                             ***CALLER MUST ASSURE that this points
//                             to a valid doubleword.***
//
// Returns:       Return code from mciSendCommand()
//
//-----------------------------------------------------------------------------
DWORD VdpGetCustomStatus ( WORD wDeviceID, DWORD dwMciFlags, DWORD *pdwReturn )
   {
   MCI_VDP_GET_MODE_PARMS  mciGetModeParms ;
   DWORD                   dwReturnCode ;

   // Set up the parm structure...
   memset ( &mciGetModeParms, 0x00, sizeof ( MCI_VDP_GET_MODE_PARMS ) ) ;

   // Do the MCI_VDP_GET_MODE call...
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_VDP_GET_MODE,
                                dwMciFlags,
                                ( DWORD ) &mciGetModeParms,
                                0 );

   // Set the return value...and return...
   *pdwReturn = mciGetModeParms.dwReturn ;
   return dwReturnCode ;
   }


//-----------------------------------------------------------------------------
// Func: VdpSonySetEnd()
//
// Desc: Sets end of disc to specified frame where specified frame is
//       less than or equal to the last accessible frame on the videodisc.
//
//       ***Note***: Be sure to include "lvdsys.h" when you use this...
//       ***Note***: Supported only with supported Sony players...
//
// Name:          VdpSonySetEnd()
//
// Description:   Sets end of disc to specified frame where specified frame is
//                less than or equal to the last accessible frame on the videodisc.
//
//                ***Note***: Be sure to include "lvdsys.h" when you use this...
//                ***Note***: Supported only with supported Sony players...
//
// Concepts:      With supported Sony players, driver cannot reliably
//                calculate last valid frame on spun up videodisc.
//                Default is frame 54000.
//
//                This may causes a problem for commands such as "seek to end"
//                or "play" that use an end-of-disc value.  If the Sony
//                seeks past the last valid frame on the disc, it "gets lost"
//                and usually resets itself to frame 1.  The driver detects
//                this problem and sets the disc to frame 1.
//
//                Application writer has the following other alternatives:
//
//                (1) Know the last valid frame, or query it from user,
//                    and use VdpSonySetEnd() to set actual-valid-frame
//                    as last frame:  for example, 43229.
//
//                (2) Do not use play, seek, or step options that use
//                    end-of-disc.
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - MCI_* flags
//
//                EndFrame - position to be used as end of frame,
//                           ***specified in the current timer format***
//
// Returns:       Return code from mciSendCommand()
//
//-----------------------------------------------------------------------------
DWORD VdpSonySetEnd ( WORD wDeviceID, DWORD dwMciFlags, DWORD EndFrame )
   {
   MCI_VDP_SET_END_PARMS  mciSetEndParms ;
   DWORD                  dwReturnCode ;

   // Set up the parm structure...
   memset ( &mciSetEndParms, 0x00, sizeof ( MCI_VDP_SET_END_PARMS ) ) ;
   mciSetEndParms.dwPosition = EndFrame ;

   // Do the MCI_VDP_SET_END call...and return...
   dwReturnCode = mciSendCommand ( wDeviceID, MCI_VDP_SET_END,
                                dwMciFlags,
                                ( DWORD ) &mciSetEndParms,
                                0 );
   return dwReturnCode ;
   }

//-----------------------------------------------------------------------------
//
// Name:          DisplayVideodiscStatus()
//
// Description:   Illustrates various ways of fetching and interpreting
//                videodisc status via MCI_STATUS calls.
//
//                ***Note***: Be sure to include "lvdsys.h" when you use this...
//
// Concepts:      Many status options, many return formats.  This function
//                tries to exemplify each format used by videodisc player device.
//
// Parameters:    wDeviceID - the device
//
// Returns:       0; displays status values in message box...
//
//-----------------------------------------------------------------------------
PSZ  apszModeStrings        [] = { "Not ready", "Paused", "Playing", "Stopped", "Recording", "Seeking" } ;
PSZ  apszGetModeStrings     [] = { "Door Open", "No Disc", "Parked", "Still" } ;
PSZ  apszTimeFormatStrings  [] = { "Milliseconds", "MMtime", "", "", "", "", "Chapters", "Frames", "HMS", "", "", "", "HMSF" }  ;
PSZ  apszSpeedFormatStrings [] = { "Percentage", "FPS" }  ;
PSZ  apszMediaTypeStrings   [] = { "CLV", "CAV" } ;
PSZ  apszBooleanStrings     [] = { "False", "True" } ;
PSZ  apszErrorStrings       [] = { "Player cannot recognize side", "Player cannot recognize size" } ;

DWORD DisplayVideodiscStatus ( WORD wDeviceID )
   {
   DWORD dwReturn ;
   DWORD dwReturnCode ;
   CHAR  szMessage [ 512 ] ;
// PCHAR psz ;
   CHAR  szNumeric [ 15 ] ;

   // Init..
   szMessage [ 0 ] = '\0' ;

   //-------------------------------------------------------------------------
   // (1) This section processes items that return MCI_MODE_RETURN,
   //     MCI_TIME_FORMAT_RETURN, MCI_SPEED_FORMAT_RETURN, or
   //     MCI_MEDIA_TYPE_RETURN
   //-------------------------------------------------------------------------
   // Get MCI_MODE...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_MODE,
                                       &dwReturn ) ;
   strcat ( szMessage, "Mode:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_MODE_RETURN...
      // The dwReturn value is MCI_MODE_NOT_READY, MCI_MODE_PAUSE,
      // MCI_MODE_PLAY, MCI_MODE_STOP, or MCI_MODE_SEEK,
      strcat ( szMessage, apszModeStrings [ dwReturn - MCI_MODE_NOT_READY ] ) ;
      }

   // Get MCI_VDP_GET_MODE...and generate text for the value...
   dwReturnCode = VdpGetCustomStatus ( ppiCurrent->wDeviceID, 0, &dwReturn ) ;
   strcat ( szMessage, "\nGetMode:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_MODE_RETURN...
      // The dwReturn value is MCI_MODE_NOT_READY, MCI_MODE_PAUSE,
      // MCI_MODE_PLAY, MCI_MODE_STOP, or MCI_MODE_SEEK...
      // or  a custom mode: MCI_VDP_MODE_DOOR_OPEN, MCI_VDP_MODE_NO_DISC,
      // MCI_VDP_MODE_PARKED, or  MCI_VDP_MODE_STILL
      if (   dwReturn >= MCI_VDP_MODE_DOOR_OPEN
          && dwReturn <= MCI_VDP_MODE_STILL )
         {
         strcat ( szMessage, apszGetModeStrings [ dwReturn - MCI_VDP_MODE_DOOR_OPEN ] ) ;
         }
      else
         {
         strcat ( szMessage, apszModeStrings [ dwReturn - MCI_MODE_NOT_READY ] ) ;
         }
      }

   // Get TIME FORMAT...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_TIME_FORMAT,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nTime Format:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_TIME_FORMAT_RETURN...
      // The dwReturn value is one of these:
      //
      //      MCI_FORMAT_MILLISECONDS
      //      MCI_FORMAT_MMTIME
      //      MCI_FORMAT_CHAPTERS
      //      MCI_FORMAT_FRAMES
      //      MCI_FORMAT_HMS
      //      MCI_FORMAT_TRACKS
      //      MCI_FORMAT_HMSF
      strcat ( szMessage, apszTimeFormatStrings [ dwReturn - MCI_FORMAT_MILLISECONDS ] ) ;
      }


   // Get SPEED FORMAT...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_SPEED_FORMAT,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nSpeed Format:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_SPEED_FORMAT_RETURN...
      // The dwReturn value is one of these:
      //
      //      MCI_FORMAT_PERCENTAGE
      //      MCI_FORMAT_FPS
      strcat ( szMessage, apszSpeedFormatStrings [ dwReturn - MCI_FORMAT_PERCENTAGE ] ) ;
      }

   // Get MEDIA TYPE...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_VD_MEDIA_TYPE,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nMedia Type:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_MEDIA_TYPE_RETURN...
      // The dwReturn value is MCI_VD_MEDIA_CAV or MCI_VD_MEDIA_CLV...
      strcat ( szMessage, apszMediaTypeStrings [ dwReturn - MCI_VD_MEDIA_CLV ] ) ;
      }


   //-------------------------------------------------------------------------
   // (2) This section processes items that return MCI_INTEGER_RETURNED,
   //     MCI_COLONIZED2_RETURN, MCI_COLONIZED3_RETURN, OR MCI_COLONIZED4_RETURN...
   //-------------------------------------------------------------------------
   //
   // Colonized format is displayed with 2, 3, or 4 numbers separated
   // by colons.  For example, hms displayed in colonized 3 format is
   // "0:30:0", representing 0 hours, 30 minutes, and 0 seconds.
   //
   // Value:               Return format:          How stored:
   //
   //
   // MCI_STATUS_VOLUME    MCI_COLONIZED2_RETURN   Low  word: left  volume
   //                                              High word: right volume
   //
   //
   // MCI_FORMAT_HMS       MCI_COLONIZED3_RETURN   byte 0 (low byte): hours
   //                                              byte 1           : minutes
   //                                              byte 2           : seconds
   //                                              byte 3           : 0
   //
   //
   // MCI_FORMAT_HMS       MCI_COLONIZED4_RETURN   byte 0 (low byte): hours
   //                                              byte 1           : minutes
   //                                              byte 2           : seconds
   //                                              byte 3           : frames
   //
   //
   // Get LENGTH...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_LENGTH,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nLength:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      //
      // The high word of the return code is MCI_INTEGER_RETURNED,
      // or MCI_COLONIZED3_RETURN, or MCI_COLONIZED4_RETURN...
      //
      // The dwReturn value is a binary value which is decoded differently
      // depending on the current time format and the value of the high
      // word of the return code, as follows:
      //
      // If the                     Then the high           And the value
      // time format                word of the             of dwReturn
      // is:                        return code is:         represents:
      //
      // MCI_FORMAT_MILLISECONDS    MCI_INTEGER_RETURNED    milliseconds
      // MCI_FORMAT_MMTIME          MCI_INTEGER_RETURNED    mmtimes
      // MCI_FORMAT_FRAMES          MCI_INTEGER_RETURNED    frames
      // MCI_FORMAT_HMS             MCI_COLONIZED3_RETURN   See Note 1
      // MCI_FORMAT_HMSF            MCI_COLONIZED4_RETURN   See Note 2
      // MCI_FORMAT_CHAPTERS        MCI_INTEGER_RETURNED    chapters
      //
      // Note 1: The 4 bytes of the dword contain (starting at the
      //         most significant byte):  frames, seconds, minutes, hours.
      //
      // Note 2: Same as Note 1, except that the "frames" value will
      //         always be 0.
      //
      //
      switch ( HIUSHORT ( dwReturnCode ) )
         {
         case MCI_INTEGER_RETURNED:
            ConvertDwordToAscii ( dwReturn, szNumeric ) ;
            break;

         case MCI_COLONIZED3_RETURN:
            // Add code to handle this...
            // Will be returned when the time format is MCI_FORMAT_HMS...
            break;

         case MCI_COLONIZED4_RETURN:
            // Add code to handle this...
            // Will be returned when the time format is MCI_FORMAT_HMSF...
            break;
         }

      strcat ( szMessage, szNumeric ) ;
      }


   // Get POSITION...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_POSITION,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nPosition:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      //
      // See comments to LENGTH above...
      //
      //
      switch ( HIUSHORT ( dwReturnCode ) )
         {
         case MCI_INTEGER_RETURNED:
            ConvertDwordToAscii ( dwReturn, szNumeric ) ;
            break;

         case MCI_COLONIZED3_RETURN:
            // Add code to handle this...
            // Will be returned by MCI when the time format is MCI_FORMAT_HMS...
            break;

         case MCI_COLONIZED4_RETURN:
            // Add code to handle this...
            // Will be returned by MCI when the time format is MCI_FORMAT_HMSF...
            break;
         }

      strcat ( szMessage, szNumeric ) ;
      }


   // Get VOLUME...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_VOLUME,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nVolume:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      //
      // See comments to Section 2 (this section) above...
      //
      //
      ConvertColonizedToAscii ( MCI_COLONIZED2_RETURN, dwReturn, szNumeric ) ;
      strcat ( szMessage, szNumeric ) ;
      }


   // Get SPEED...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_VD_STATUS_SPEED,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nSpeed:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      //
      // The low  word of the return code is MCIERR_SUCCESS...
      //
      // The high word of the return code is MCI_INTEGER_RETURNED...
      //
      // The dwReturn value is a binary integer.
      //
      ConvertDwordToAscii ( dwReturn, szNumeric ) ;
      strcat ( szMessage, szNumeric ) ;
      }


   // Get DISC SIDE...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_VD_STATUS_SIDE,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nDisc Side:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      //
      // The low  word of the return code is MCIERR_SUCCESS...
      //
      // The high word of the return code is MCI_INTEGER_RETURNED...
      //
      // The dwReturn value is a binary integer.
      //
      ConvertDwordToAscii ( dwReturn, szNumeric ) ;
      strcat ( szMessage, szNumeric ) ;
      }

   else if ( LOUSHORT ( dwReturnCode ) == MCIERR_VDP_NOSIDE )
      {
      strcat ( szMessage, apszErrorStrings [ 0 ] ) ;
      }


   // Get DISC SIZE...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_VD_STATUS_DISC_SIZE,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nDisc Size:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      //
      // The low  word of the return code is MCIERR_SUCCESS...
      //
      // The high word of the return code is MCI_INTEGER_RETURNED...
      //
      // The dwReturn value is a binary integer.
      //
      ConvertDwordToAscii ( dwReturn, szNumeric ) ;
      strcat ( szMessage, szNumeric ) ;
      }

   else if ( LOUSHORT ( dwReturnCode ) == MCIERR_VDP_NOSIZE )
      {
      strcat ( szMessage, apszErrorStrings [ 1 ] ) ;
      }


   //-------------------------------------------------------------------------
   // (3) This section processes items that return MCI_TRUE_FALSE_RETURN
   //
   //-------------------------------------------------------------------------
   // Get MEDIA PRESENT...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_MEDIA_PRESENT,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nMedia Present:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_TRUE_FALSE_RETURNED...
      // The dwReturn value is either MCI_TRUE or MCI_FALSE...
      strcat ( szMessage, apszBooleanStrings [ dwReturn - MCI_FALSE ] ) ;
      }


   // Get READY...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_STATUS_READY,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nReady:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_TRUE_FALSE_RETURNED...
      // The dwReturn value is either MCI_TRUE or MCI_FALSE...
      strcat ( szMessage, apszBooleanStrings [ dwReturn - MCI_FALSE ] ) ;
      }


   // Get FORWARD...and generate text for the value...
   dwReturnCode = VdpGetStatus ( ppiCurrent->wDeviceID,
                                       MCI_STATUS_ITEM,
                                       MCI_VD_STATUS_FORWARD,
                                       &dwReturn ) ;
   strcat ( szMessage, "\nForward:   " ) ;
   if ( LOUSHORT ( dwReturnCode ) == MCIERR_SUCCESS )
      {
      // The low  word of the return code is MCIERR_SUCCESS...
      // The high word of the return code is MCI_TRUE_FALSE_RETURNED...
      // The dwReturn value is either MCI_TRUE or MCI_FALSE...
      strcat ( szMessage, apszBooleanStrings [ dwReturn - MCI_FALSE ] ) ;
      }


   //-------------------------------------------------------------------------
   // (4) Now display the message in a message box...
   //
   //-------------------------------------------------------------------------
   // Display...
   WinMessageBox ( HWND_DESKTOP, HWND_DESKTOP, ( PCH ) szMessage,
      ( PCH ) "Player Status Values", ID_MESSAGEBOX,
      MB_OK | MB_APPLMODAL | MB_MOVEABLE );

   // Done...
   return 0 ;
   }

//-----------------------------------------------------------------------------
// Func: VdpSetAudio()
//
// Desc: Sets player audio on or off.
//
// Name:          VdpSetAudio()
//
// Description:   Sets player audio on or off.
//
// Concepts:      Very quick command.
//
// Parameters:    wDeviceID - the device
//
//                dwMciFlags - flags
//
//                dwAudio -  MCI_SET_AUDIO_ALL, MCI_SET_AUDIO_LEFT,
//                           or MCI_SET_AUDIO_RIGHT
//
// Returns:       FALSE if successful, else TRUE
//
//-----------------------------------------------------------------------------
DWORD VdpSetAudio ( WORD wDeviceID, DWORD dwMciFlags, DWORD dwAudio )
   {
   MCI_VD_SET_PARMS  mciVdSetParms ;
   DWORD   dwReturnCode ;

   // Note:  If the master audio is set to 0, the videodisc player driver
   //        will NOT set the videodisc player audio on....

   // Send an MCI_SET command to switch the audio on or off...
   memset ( &mciVdSetParms, 0x00, sizeof ( MCI_VD_SET_PARMS ) ) ;
   mciVdSetParms.dwAudio    = dwAudio ;
   dwReturnCode = mciSendCommand( wDeviceID, MCI_SET,
                              dwMciFlags,
                              ( DWORD )&mciVdSetParms,
                              0 );

   return (LOUSHORT(dwReturnCode) != MCIERR_SUCCESS);
   }


//-----------------------------------------------------------------------------
//
// Name:          ConvertDwordToAscii()
//
// Description:   Looks a lot like ultoa(), probably shd have used that instead..
//
// Returns:       Number of characters in output string excluding the '\0'...
//
//-----------------------------------------------------------------------------
DWORD ConvertDwordToAscii ( DWORD dwValue, PBYTE pbString )
   {
   PBYTE pb ;
   DWORD dwDivisor ;
   int   i ;
   BYTE  abTemp [ 12 ] ;

   // Init...
   dwDivisor = 10000000L ;

   // Build ascii string representation of number in temporary buffer...
   for  ( i = 0, pb = abTemp ; i < 8; i++, pb++ )
      {
      *pb = ( ( BYTE ) ( dwValue / dwDivisor ) ) + ( BYTE ) '0' ;
      dwValue = dwValue  % dwDivisor ;
      dwDivisor = dwDivisor / 10 ;
      }

   // Make it a string...
   *pb = '\0' ;

   // Skip leading 0's
   for ( i = 0, pb = abTemp ; i < 7 ; i++, pb++ )
      {
      if ( *pb != '0' )
         {
         break ;
         }
      }

   // Copy to output buffer...and return
   strcpy ( pbString, pb ) ;
   return strlen ( pbString ) ;
   }


//-----------------------------------------------------------------------------
//
// Name:          ConvertColonizedToAscii
//
// Description:   Number of characters in output string excluding the '\0'...
//
// Concepts:      Useful for converting colonized format to ascii
//
// Parameters:    dwFormat - which form of colonized
//
//                dwColonized - colonized input
//
//                pbString - string output...
//
// Returns:       Length of output string.
//
//-----------------------------------------------------------------------------
DWORD ConvertColonizedToAscii ( DWORD dwFormat, DWORD dwColonized, PBYTE pbString )
   {
   BYTE     abTemp [ 64 ] ;
   PBYTE    pb = abTemp ;
   DWORD    dwLength ;

   // If COLONIZED2...
   if ( dwFormat == MCI_COLONIZED2_RETURN )
      {
      // Convert the low word...
      dwLength = ConvertDwordToAscii ( LOUSHORT ( dwColonized ), pb ) ;
      pb      += dwLength ;
      *(pb++)   = ':' ;

      // Convert the high word...
      dwLength = ConvertDwordToAscii ( HIUSHORT ( dwColonized ), pb ) ;
      pb      += dwLength ;
      }

   // Else assume COLONIZED3 or COLONIZED4...
   else
      {

      // Convert byte 0 (low byte)...
      dwLength = ConvertDwordToAscii ( DWORD_LWLB ( dwColonized ), pb ) ;
      pb      += dwLength ;

      // Store a ':' and convert byte 1...
      *(pb++)   = ':' ;
      dwLength  = ConvertDwordToAscii ( DWORD_LWHB ( dwColonized ), pb ) ;
      pb       += dwLength ;

      // Store a ':' and convert byte 2...
      *(pb++)  = ':' ;
      dwLength = ConvertDwordToAscii ( DWORD_HWLB ( dwColonized ), pb ) ;
      pb      += dwLength ;

      // If COLONIZED4 then...store a ':' and convert byte 3...
      if ( dwFormat == MCI_COLONIZED4_RETURN )
         {
         *(pb++)  = ':' ;
         dwLength = ConvertDwordToAscii ( DWORD_HWHB ( dwColonized ), pb ) ;
         pb      += dwLength ;
         }
      }

   // Terminate the string...
   *pb = '\0';

   // Copy to output buffer...and return
   strcpy ( pbString, abTemp ) ;
   return strlen ( pbString ) ;
   }


//*****************************************************************************
// End of file
//*****************************************************************************
